__author__ = 'churas'

import unittest
import tempfile
import stat
import re
import tarfile
from mock import Mock

from d3r.celpp.filetransfer import FtpFileTransfer

"""
test_proteinligprep
--------------------------------

Tests for `proteinligprep` module.
"""

import shutil
import os
from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.blastnfilter import BlastNFilterTask
from d3r.celpp.challengedata import ChallengeDataTask
from d3r.celpp.dataimport import DataImportTask


class TestChallengeDataTask(unittest.TestCase):
    def setUp(self):
        pass

    def test_get_uploadable_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # test where directory doesn't even exist
            params = D3RParameters()
            task = ChallengeDataTask(temp_dir, params)
            self.assertEqual(task.get_uploadable_files(), [])

            # test on empty dir
            task.create_dir()
            self.assertEqual(task.get_uploadable_files(), [])

            # test with tarfile
            tarfile = task.get_celpp_challenge_data_tar_file()

            open(tarfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 1)
            flist.index(tarfile)

            # test with additional stderr/stdout files
            errfile = os.path.join(task.get_dir(),
                                   'genchallengedata.py.stderr')
            open(errfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            flist.index(errfile)

            outfile = os.path.join(task.get_dir(),
                                   'genchallengedata.py.stdout')
            open(outfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 3)
            flist.index(outfile)
            flist.index(errfile)
            flist.index(tarfile)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_celpp_challenge_data_dir_name_path_not_found(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ChallengeDataTask(temp_dir, params)
            self.assertEqual(task.get_celpp_challenge_data_dir_name(),
                             'celpp_week0_0')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_celpp_challenge_data_dir_name_not_standard_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ChallengeDataTask(temp_dir, params)
            task.create_dir()
            self.assertEqual(task.get_celpp_challenge_data_dir_name(),
                             'celpp_week0_0')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_celpp_challenge_data_dir_name(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            cyear = os.path.join(temp_dir, '2016')
            cweek = os.path.join(cyear, 'dataset.week.5')
            os.mkdir(cyear)
            os.mkdir(cweek)
            task = ChallengeDataTask(cweek, params)
            task.create_dir()
            self.assertEqual(task.get_celpp_challenge_data_dir_name(),
                             'celpp_week5_2016')

        finally:
            shutil.rmtree(temp_dir)

    def test_create_challenge_dir_unable_to_make_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            cyear = os.path.join(temp_dir, '2016')
            cweek = os.path.join(cyear, 'dataset.week.5')
            os.mkdir(cyear)
            os.mkdir(cweek)
            task = ChallengeDataTask(cweek, params)
            task.create_dir()
            cdir = os.path.join(task.get_dir(),
                                task.get_celpp_challenge_data_dir_name())
            open(cdir, 'a').close()

            try:
                task._create_challenge_dir()
                self.fail('Expected OSError')
            except OSError:
                pass

        finally:
            shutil.rmtree(temp_dir)

    def test_create_challenge_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            cyear = os.path.join(temp_dir, '2016')
            cweek = os.path.join(cyear, 'dataset.week.5')
            os.mkdir(cyear)
            os.mkdir(cweek)
            task = ChallengeDataTask(cweek, params)
            task.create_dir()
            cdir = os.path.join(task.get_dir(),
                                task.get_celpp_challenge_data_dir_name())
            task._create_challenge_dir()
            self.assertEqual(os.path.isdir(cdir), True)
            task._create_challenge_dir()
            self.assertEqual(os.path.isdir(cdir), True)

        finally:
            shutil.rmtree(temp_dir)

    def test_create_readme_unable_to_write_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.version = "1.0.0"
            task = ChallengeDataTask(temp_dir, params)
            try:
                task._create_readme(os.path.join(temp_dir, 'doesnotexist'))
                self.fail('Expected IOError')
            except IOError:
                pass
        finally:
            shutil.rmtree(temp_dir)

    def test_create_readme_version_unset_and_no_summary_txt(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ChallengeDataTask(temp_dir, params)
            task._create_readme(temp_dir)
            readme = os.path.join(temp_dir,
                                  ChallengeDataTask.README_TXT_FILE)
            self.assertEqual(os.path.isfile(readme), True)
            f = open(readme, 'r')
            found = False
            for line in f:

                if re.match('^Celpprunner version: Unknown.*$', line):
                    found = True
                    break

            f.close()
            self.assertEqual(found, True)

        finally:
            shutil.rmtree(temp_dir)

    def test_create_readme_version_empty_summary(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.version = '1.0'
            task = ChallengeDataTask(temp_dir, params)
            blast = BlastNFilterTask(temp_dir, params)
            blast.create_dir()
            sfile = blast.get_blastnfilter_summary_file()
            open(sfile, 'a').close()

            task._create_readme(temp_dir)
            readme = os.path.join(temp_dir,
                                  ChallengeDataTask.README_TXT_FILE)
            self.assertEqual(os.path.isfile(readme), True)
            f = open(readme, 'r')
            found = False
            for line in f:

                if re.match('^Celpprunner version: 1.0.*$', line):
                    found = True
                    break

            f.close()
            self.assertEqual(found, True)

        finally:
            shutil.rmtree(temp_dir)

    def test_create_readme(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.version = '1.0'
            task = ChallengeDataTask(temp_dir, params)
            blast = BlastNFilterTask(temp_dir, params)
            blast.create_dir()
            sfile = blast.get_blastnfilter_summary_file()
            f = open(sfile, 'w')
            f.write('hello there\n')
            f.flush()
            f.close()

            task._create_readme(temp_dir)
            readme = os.path.join(temp_dir,
                                  ChallengeDataTask.README_TXT_FILE)
            self.assertEqual(os.path.isfile(readme), True)
            f = open(readme, 'r')
            found = False
            for line in f:
                print line
                if re.match('^hello there.*$', line):
                    found = True
                    break

            f.close()
            self.assertEqual(found, True)

        finally:
            shutil.rmtree(temp_dir)

    def test_copy_over_tsv_files_no_dataimport(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ChallengeDataTask(temp_dir, params)
            task.create_dir()
            challenge_dir = task._create_challenge_dir()
            self.assertEqual(os.path.isdir(challenge_dir), True)
            task._copy_over_tsv_files(challenge_dir)

        finally:
            shutil.rmtree(temp_dir)

    def test_copy_over_tsv_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            dimport = DataImportTask(temp_dir, params)
            dimport.create_dir()

            ctsv = dimport.get_crystalph_tsv()
            f = open(ctsv, 'w')
            f.write('crystal')
            f.flush()
            f.close()

            nonpoly = dimport.get_nonpolymer_tsv()
            f = open(nonpoly, 'w')
            f.write('nonpoly')
            f.flush()
            f.close()

            seq = dimport.get_sequence_tsv()
            f = open(seq, 'w')
            f.write('seq')
            f.flush()
            f.close()

            task = ChallengeDataTask(temp_dir, params)
            task.create_dir()
            challenge_dir = task._create_challenge_dir()
            self.assertEqual(os.path.isdir(challenge_dir), True)
            task._copy_over_tsv_files(challenge_dir)

            cop_ctsv = os.path.join(challenge_dir,
                                    DataImportTask.CRYSTALPH_TSV)
            self.assertEqual(os.path.isfile(cop_ctsv), True)
            f = open(cop_ctsv)
            self.assertEqual(f.readline(), 'crystal')
            f.close()

            cop_nonpoly = os.path.join(challenge_dir,
                                       DataImportTask.NONPOLYMER_TSV)
            self.assertEqual(os.path.isfile(cop_nonpoly), True)
            f = open(cop_nonpoly)
            self.assertEqual(f.readline(), 'nonpoly')
            f.close()

            cop_seq = os.path.join(challenge_dir,
                                   DataImportTask.SEQUENCE_TSV)
            self.assertEqual(os.path.isfile(cop_seq), True)
            f = open(cop_seq)
            self.assertEqual(f.readline(), 'seq')
            f.close()

        finally:
            shutil.rmtree(temp_dir)

    def test_tar_challenge_dir_unable_to_write_tarfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            task = ChallengeDataTask(temp_dir, params)

            chall_dir = task.get_celpp_challenge_data_dir_name()
            try:
                task._tar_challenge_dir(chall_dir)
                self.fail('Expected IOError')
            except IOError:
                pass

        finally:
            shutil.rmtree(temp_dir)

    def test_tar_challenge_dir_nothing_but_readme(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ChallengeDataTask(temp_dir, params)
            task.create_dir()
            chall_dir = task._create_challenge_dir()
            task._create_readme(chall_dir)

            self.assertEqual(os.path.isdir(chall_dir), True)
            name = task.get_celpp_challenge_data_dir_name()
            tfile = task._tar_challenge_dir(name)
            self.assertEqual(os.path.isfile(tfile), True)

            foodir = os.path.join(temp_dir, 'foo')
            tar = tarfile.open(tfile, 'r:*')
            tar.extractall(path=foodir)
            tar.close()
            cdir = os.path.join(foodir, name)
            readme = os.path.join(cdir, ChallengeDataTask.README_TXT_FILE)
            self.assertEqual(os.path.isfile(readme), True)

        finally:
            shutil.rmtree(temp_dir)

    def make_fake_candidate_dir(self, basedir, targetid,
                                candidateid, candidateligandid):
        """creates fake candidate dir
        """
        namedir = os.path.join(basedir, targetid)
        os.mkdir(namedir)
        file_list = []
        fasta = os.path.join(namedir, targetid + '.fasta')
        self.write_fake_data_to_file(fasta)
        file_list.append(fasta.replace(basedir + '/', ''))
        txt = os.path.join(namedir, targetid + '.txt')
        self.write_fake_data_to_file(txt)
        file_list.append(txt.replace(basedir + '/', ''))
        largest = os.path.join(namedir,
                               self.generate_pdb_file_name('largest',
                                                           targetid,
                                                           candidateid,
                                                           candidateligandid))
        self.write_fake_data_to_file(largest)
        file_list.append(largest.replace(basedir + '/', ''))
        smallest = os.path.join(namedir,
                                self.generate_pdb_file_name('smallest',
                                                            targetid,
                                                            candidateid,
                                                            candidateligandid))
        self.write_fake_data_to_file(smallest)
        file_list.append(smallest.replace(basedir + '/', ''))
        apo = os.path.join(namedir,
                           self.generate_pdb_file_name('apo',
                                                       targetid,
                                                       candidateid,
                                                       candidateligandid))
        self.write_fake_data_to_file(apo)
        file_list.append(apo.replace(basedir + '/', ''))

        holo = os.path.join(namedir,
                            self.generate_pdb_file_name('holo',
                                                        targetid,
                                                        candidateid,
                                                        candidateligandid))
        self.write_fake_data_to_file(holo)
        file_list.append(holo.replace(basedir + '/', ''))

        lig_smi = os.path.join(namedir, 'lig_' + candidateligandid +
                               '.smi')
        self.write_fake_data_to_file(lig_smi)
        file_list.append(lig_smi.replace(basedir + '/', ''))

        lig_inchi = os.path.join(namedir, 'lig_' + candidateligandid +
                                 '.inchi')
        self.write_fake_data_to_file(lig_inchi)
        file_list.append(lig_inchi.replace(basedir + '/', ''))

        lig_mol = os.path.join(namedir, 'lig_' + candidateligandid +
                               '.mol')
        self.write_fake_data_to_file(lig_mol)
        file_list.append(lig_mol.replace(basedir + '/', ''))
        return file_list

    def write_fake_data_to_file(self, filepath):
        """Writes fake data into file passed in

        The data content is the name of the file
        as extracted from os.path.basename(filepath)
        :param filepath: path to file
        """
        f = open(filepath, 'w')
        f.write(os.path.basename(filepath))
        f.flush()
        f.close()

    def generate_pdb_file_name(self, type, targetid, candidateid,
                               candidateligandid):
        """Generates a pdb file name matching challenge data dir format
        """
        return (type + '-' + targetid + '_' + candidateid + '_' +
                candidateligandid + '.pdb')

    def test_tar_challenge_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            dimport = DataImportTask(temp_dir, params)
            dimport.create_dir()
            open(dimport.get_sequence_tsv(), 'a').close()
            open(dimport.get_nonpolymer_tsv(), 'a').close()
            open(dimport.get_crystalph_tsv(), 'a').close()

            task = ChallengeDataTask(temp_dir, params)
            task.create_dir()
            chall_dir = task._create_challenge_dir()

            final_log = os.path.join(chall_dir, 'final.log')
            open(final_log, 'a').close()

            task._create_readme(chall_dir)
            task._copy_over_tsv_files(chall_dir)

            # make a fake candidate
            file_list = self.make_fake_candidate_dir(chall_dir, '5hib',
                                                     '2eb2', 'CSX')

            self.assertEqual(os.path.isdir(chall_dir), True)
            name = task.get_celpp_challenge_data_dir_name()
            tfile = task._tar_challenge_dir(name)
            self.assertEqual(os.path.isfile(tfile), True)

            foodir = os.path.join(temp_dir, 'foo')
            tar = tarfile.open(tfile, 'r:*')
            tar.extractall(path=foodir)
            tar.close()
            cdir = os.path.join(foodir, name)
            readme = os.path.join(cdir, ChallengeDataTask.README_TXT_FILE)
            self.assertEqual(os.path.isfile(readme), True)

            final_log = os.path.join(foodir, 'final.log')
            self.assertEqual(os.path.isfile(final_log), False)

            for fname in file_list:
                chk = os.path.join(cdir, fname)
                self.assertEqual(os.path.isfile(chk), True, chk)

        finally:
            shutil.rmtree(temp_dir)

    def test_can_run(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # no blast task found so it cannot run
            params = D3RParameters()
            task = ChallengeDataTask(temp_dir, params)
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             'blastnfilter task has notfound status')

            # blastn filter running
            blastnfilter = BlastNFilterTask(temp_dir, params)
            blastnfilter.create_dir()
            open(os.path.join(blastnfilter.get_dir(), D3RTask.START_FILE),
                 'a').close()
            task = ChallengeDataTask(temp_dir, params)
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             'blastnfilter task has start status')

            # blastnfilter failed
            error_file = os.path.join(blastnfilter.get_dir(),
                                      D3RTask.ERROR_FILE)
            open(error_file, 'a').close()
            task = ChallengeDataTask(temp_dir, params)
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             'blastnfilter task has error status')

            # blastnfilter success
            os.remove(error_file)
            open(os.path.join(blastnfilter.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            task = ChallengeDataTask(temp_dir, params)
            self.assertEqual(task.can_run(), True)
            self.assertEqual(task.get_error(), None)

            # proteinligprep task exists already
            task = ChallengeDataTask(temp_dir, params)
            task.create_dir()
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             task.get_dir_name() +
                             ' already exists and status is unknown')

            # proteinlibprep already complete
            task = ChallengeDataTask(temp_dir, params)
            open(os.path.join(task.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(), None)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_can_run_is_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.version = '1'
            # return immediately cause can_run is false
            chall = ChallengeDataTask(temp_dir, params)
            chall.run()
            self.assertEqual(chall.get_error(),
                             'blastnfilter task has notfound status')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_genchallenge_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.version = '1'
            blastnfilter = BlastNFilterTask(temp_dir, params)
            blastnfilter.create_dir()
            open(os.path.join(blastnfilter.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            chall = ChallengeDataTask(temp_dir, params)
            chall.run()
            self.assertEqual(chall.get_error(),
                             'genchallenge not set')
            # test files get created
            self.assertEqual(os.path.isdir(chall.get_dir()),
                             True)
            errfile = os.path.join(chall.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_pdbdb_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.genchallenge = 'false'
            params.version = '1'
            blastnfilter = BlastNFilterTask(temp_dir, params)
            blastnfilter.create_dir()
            open(os.path.join(blastnfilter.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            chall = ChallengeDataTask(temp_dir, params)
            chall.run()
            self.assertEqual(chall.get_error(),
                             'pdbdb not set')
            # test files get created
            self.assertEqual(os.path.isdir(chall.get_dir()),
                             True)
            errfile = os.path.join(chall.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_genchallenge_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.genchallenge = 'false'
            params.pdbdb = '/foo'
            params.version = '1'
            blastnfilter = BlastNFilterTask(temp_dir, params)
            blastnfilter.create_dir()
            open(os.path.join(blastnfilter.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            chall = ChallengeDataTask(temp_dir, params)

            chall.run()
            self.assertEqual(chall.get_error(),
                             'Non zero exit code: 1 received. Standard out: ' +
                             ' Standard error: ')
            # test file gets created
            errfile = os.path.join(chall.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)

            stderr = os.path.join(chall.get_dir(),
                                  'false.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(chall.get_dir(),
                                  'false.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_genchallenge_is_not_found(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.genchallenge = '/bin/doesnotexist'
            params.pdbdb = '/foo'
            params.version = '1'
            blastnfilter = BlastNFilterTask(temp_dir, params)
            blastnfilter.create_dir()
            open(os.path.join(blastnfilter.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            chall = ChallengeDataTask(temp_dir, params)

            chall.run()
            self.assertEqual(chall.get_error(),
                             'Caught Exception trying to run ' +
                             '/bin/doesnotexist --candidatedir ' +
                             blastnfilter.get_dir() + ' --pdbdb ' +
                             '/foo --outdir ' +
                             chall.get_dir() +
                             '/' + chall.get_celpp_challenge_data_dir_name() +
                             ' : [Errno 2] No such file or directory')

            # test files get created
            errfile = os.path.join(chall.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_challenge_file_challenge_file_is_none(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            chall = ChallengeDataTask(temp_dir, params)
            chall._upload_challenge_file(None)
            chall.set_file_transfer(params)
            self.assertEqual(chall.get_email_log(), 'challenge_file is None in'
                                                    ' _upload_challenge_file'
                                                    '\n')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_challenge_file_uploader_is_none(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            chall = ChallengeDataTask(temp_dir, params)
            chall._upload_challenge_file('/foo')
            self.assertEqual(chall.get_email_log(), 'No uploader available '
                                                    'to upload challenge '
                                                    'data\n')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_challenge_file_no_remote_challenge_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            chall = ChallengeDataTask(temp_dir, params)
            ftp = FtpFileTransfer(None)
            chall.set_file_transfer(ftp)
            chall._upload_challenge_file('/foo')
            self.assertEqual(chall.get_email_log(), 'No remote challenge'
                                                    ' directory set for '
                                                    'ftp upload\n')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_challenge_file_uploader_upload_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            yeardir = os.path.join(temp_dir, '2016')
            os.mkdir(yeardir)
            weekdir = os.path.join(yeardir, 'dataset.week.50')
            os.mkdir(weekdir)
            chall = ChallengeDataTask(weekdir, params)
            chall.create_dir()

            mockftp = D3RParameters()
            mockftp.put = Mock(side_effect=IOError('hi'))
            ftp = FtpFileTransfer(None)
            ftp.set_remote_challenge_dir('/challenge')
            ftp.set_connection(mockftp)
            ftp.connect()
            chall.set_file_transfer(ftp)
            tarball = os.path.join(chall.get_dir(), 'celppweek50_2016.tar.gz')
            f = open(tarball, 'w')
            f.write('hi')
            f.flush()
            f.close()
            try:
                chall._upload_challenge_file(tarball)
                self.fail('Expected exception')
            except Exception as e:
                self.assertEqual(str(e),
                                 'Unable to upload ' + tarball +
                                 ' to /challenge/celppweek50_2016.tar.gz : ' +
                                 'hi')
            ftp.disconnect()
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_challenge_file_uploader_tar_succeeds_latest_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            yeardir = os.path.join(temp_dir, '2016')
            os.mkdir(yeardir)
            weekdir = os.path.join(yeardir, 'dataset.week.50')
            os.mkdir(weekdir)
            chall = ChallengeDataTask(weekdir, params)
            chall.create_dir()

            mockftp = D3RParameters()
            mockftp.put = Mock(side_effect=[3, IOError('hi')])
            ftp = FtpFileTransfer(None)
            ftp.set_remote_challenge_dir('/challenge')
            ftp.set_connection(mockftp)
            ftp.connect()
            chall.set_file_transfer(ftp)
            tarball = os.path.join(chall.get_dir(), 'celppweek50_2016.tar.gz')
            f = open(tarball, 'w')
            f.write('hi')
            f.flush()
            f.close()
            latest_file = os.path.join(chall.get_dir(),
                                       ChallengeDataTask.LATEST_TXT)
            try:
                chall._upload_challenge_file(tarball)
                self.fail('Expected exception')
            except Exception as e:
                self.assertEqual(str(e),
                                 'Unable to upload ' + latest_file +
                                 ' to /challenge/latest.txt : ' +
                                 'hi')

            f = open(latest_file, 'r')
            line = f.readline()
            self.assertEqual(line, 'celppweek50_2016.tar.gz')
            f.close()
            ftp.disconnect()
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_challenge_file_uploader_successful(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            yeardir = os.path.join(temp_dir, '2017')
            os.mkdir(yeardir)
            weekdir = os.path.join(yeardir, 'dataset.week.1')
            os.mkdir(weekdir)
            chall = ChallengeDataTask(weekdir, params)
            chall.create_dir()

            mockftp = D3RParameters()
            mockftp.put = Mock(side_effect=[3, 5])
            ftp = FtpFileTransfer(None)
            ftp.set_remote_challenge_dir('/challenge')
            ftp.set_connection(mockftp)
            ftp.connect()
            chall.set_file_transfer(ftp)
            tarball = os.path.join(chall.get_dir(), 'celppweek1_2017.tar.gz')
            f = open(tarball, 'w')
            f.write('hi')
            f.flush()
            f.close()
            latest_file = os.path.join(chall.get_dir(),
                                       ChallengeDataTask.LATEST_TXT)
            chall._upload_challenge_file(tarball)

            f = open(latest_file, 'r')
            line = f.readline()
            self.assertEqual(line, 'celppweek1_2017.tar.gz')
            f.close()
            ftp.disconnect()
        finally:
            shutil.rmtree(temp_dir)

    def create_gen_challenge_script(self, temp_dir):
        script = os.path.join(temp_dir, 'genchallenge.py')
        f = open(script, 'w')
        f.write("""#! /usr/bin/env python
import argparse
import sys
import os

class D3RParameters(object):
    pass

parsed_arguments = D3RParameters()
desc = 'hi'
help_formatter = argparse.RawDescriptionHelpFormatter
parser = argparse.ArgumentParser(description=desc,
                                 formatter_class=help_formatter)
parser.add_argument("--outdir", help='blah')
parser.add_argument("--candidatedir", help='blah')
parser.add_argument("--pdbdb", help='blah')

theargs = parser.parse_args(sys.argv[1:], namespace=parsed_arguments)

open(os.path.join(theargs.outdir,'final.log'), 'a').close()

thedir = os.path.join(theargs.outdir, 'error_container')
os.mkdir(thedir)
open(os.path.join(thedir,'foo.txt'),'a').close()

thedir = os.path.join(theargs.outdir, '5hib')
os.mkdir(thedir)

open(os.path.join(thedir,'5hib.fasta'),'a').close()
open(os.path.join(thedir,'5hib.txt'),'a').close()
open(os.path.join(thedir,'apo-5hib_2eb2.pdb'),'a').close()
open(os.path.join(thedir,'holo-5hib_5hg8-634.pdb'),'a').close()
open(os.path.join(thedir,'largest-5hib_5em6-5Q3.pdb'),'a').close()
open(os.path.join(thedir,'smallest-5hib_4wrg-CSX.pdb'),'a').close()
open(os.path.join(thedir,'lig_63M.inchi'),'a').close()
open(os.path.join(thedir,'lig_63M.mol'),'a').close()
open(os.path.join(thedir,'lig_63M.smi'),'a').close()

thedir = os.path.join(theargs.outdir, '5hic')
os.mkdir(thedir)
open(os.path.join(thedir,'5hic.fasta'),'a').close()
open(os.path.join(thedir,'5hic.txt'),'a').close()
open(os.path.join(thedir,'apo-5hic_2eb2.pdb'),'a').close()
open(os.path.join(thedir,'holo-5hic_5hg8-634.pdb'),'a').close()
open(os.path.join(thedir,'largest-5hic_5em6-5Q3.pdb'),'a').close()
open(os.path.join(thedir,'smallest-5hic_4wrg-CSX.pdb'),'a').close()
open(os.path.join(thedir,'lig_63N.inchi'),'a').close()
open(os.path.join(thedir,'lig_63N.mol'),'a').close()
open(os.path.join(thedir,'lig_63N.smi'),'a').close()

""")
        f.flush()
        f.close()
        os.chmod(script, stat.S_IRWXU)
        return script

    def test_run_fails_cause_ftp_upload_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            script = self.create_gen_challenge_script(temp_dir)
            params = D3RParameters()
            params.genchallenge = script
            params.pdbdb = '/foo'
            params.version = '1'

            blastnfilter = BlastNFilterTask(temp_dir, params)
            blastnfilter.create_dir()
            open(os.path.join(blastnfilter.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()

            chall = ChallengeDataTask(temp_dir, params)
            mockftp = D3RParameters()
            mockftp.put = Mock(side_effect=[3, IOError('hi')])
            ftp = FtpFileTransfer(None)
            ftp.set_remote_challenge_dir('/challenge')
            ftp.set_connection(mockftp)
            ftp.connect()
            chall.set_file_transfer(ftp)

            dimport = DataImportTask(temp_dir, params)
            dimport.create_dir()

            ctsv = dimport.get_crystalph_tsv()
            f = open(ctsv, 'w')
            f.write('crystal')
            f.flush()
            f.close()

            nonpoly = dimport.get_nonpolymer_tsv()
            f = open(nonpoly, 'w')
            f.write('nonpoly')
            f.flush()
            f.close()

            seq = dimport.get_sequence_tsv()
            f = open(seq, 'w')
            f.write('seq')
            f.flush()
            f.close()

            chall.run()
            latest_file = os.path.join(chall.get_dir(),
                                       ChallengeDataTask.LATEST_TXT)
            self.assertEqual(chall.get_error(), 'Caught exception ' +
                             'Unable to upload ' + latest_file + ' to ' +
                             '/challenge/' + ChallengeDataTask.LATEST_TXT +
                             ' : hi')
            # verify test files get created
            errfile = os.path.join(chall.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)

            compfile = os.path.join(chall.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), False)
            ftp.disconnect()
        finally:
            shutil.rmtree(temp_dir)

    def test_run_success_with_ftp_upload(self):
        temp_dir = tempfile.mkdtemp()
        try:
            script = self.create_gen_challenge_script(temp_dir)
            params = D3RParameters()
            params.genchallenge = script
            params.pdbdb = '/foo'
            params.version = '1'

            blastnfilter = BlastNFilterTask(temp_dir, params)
            blastnfilter.create_dir()
            open(os.path.join(blastnfilter.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()

            chall = ChallengeDataTask(temp_dir, params)
            mockftp = D3RParameters()
            mockftp.put = Mock(side_effect=[3, 5])
            ftp = FtpFileTransfer(None)
            ftp.set_remote_challenge_dir('/challenge')
            ftp.set_connection(mockftp)
            chall.set_file_transfer(ftp)

            dimport = DataImportTask(temp_dir, params)
            dimport.create_dir()

            ctsv = dimport.get_crystalph_tsv()
            f = open(ctsv, 'w')
            f.write('crystal')
            f.flush()
            f.close()

            nonpoly = dimport.get_nonpolymer_tsv()
            f = open(nonpoly, 'w')
            f.write('nonpoly')
            f.flush()
            f.close()

            seq = dimport.get_sequence_tsv()
            f = open(seq, 'w')
            f.write('seq')
            f.flush()
            f.close()

            chall.run()
            self.assertEqual(chall.get_error(), None)
            # verify test files get created
            errfile = os.path.join(chall.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), False)

            compfile = os.path.join(chall.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)
            ftp.disconnect()

        finally:
            shutil.rmtree(temp_dir)

    def test_run_succeeds_no_ftp(self):
        temp_dir = tempfile.mkdtemp()
        try:
            script = self.create_gen_challenge_script(temp_dir)
            params = D3RParameters()
            params.genchallenge = script
            params.pdbdb = '/foo'
            params.version = '1'

            blastnfilter = BlastNFilterTask(temp_dir, params)
            blastnfilter.create_dir()
            open(os.path.join(blastnfilter.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            chall = ChallengeDataTask(temp_dir, params)

            dimport = DataImportTask(temp_dir, params)
            dimport.create_dir()

            ctsv = dimport.get_crystalph_tsv()
            f = open(ctsv, 'w')
            f.write('crystal')
            f.flush()
            f.close()

            nonpoly = dimport.get_nonpolymer_tsv()
            f = open(nonpoly, 'w')
            f.write('nonpoly')
            f.flush()
            f.close()

            seq = dimport.get_sequence_tsv()
            f = open(seq, 'w')
            f.write('seq')
            f.flush()
            f.close()

            chall.run()
            self.assertEqual(chall.get_error(), None)
            # verify test files get created
            errfile = os.path.join(chall.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), False)

            compfile = os.path.join(chall.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)
            stderr = os.path.join(chall.get_dir(),
                                  'genchallenge.py.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(chall.get_dir(),
                                  'genchallenge.py.stdout')
            self.assertEqual(os.path.isfile(stdout), True)

            # verify challenge directory is created and
            # filled with valid files
            chall_dir = os.path.join(chall.get_dir(),
                                     chall.get_celpp_challenge_data_dir_name())
            self.assertEqual(os.path.isdir(chall_dir), True)
            readme = os.path.join(chall_dir, ChallengeDataTask.README_TXT_FILE)
            self.assertEqual(os.path.isfile(readme), True)

            crystal = os.path.join(chall_dir, DataImportTask.CRYSTALPH_TSV)
            self.assertEqual(os.path.isfile(crystal), True)

            seq = os.path.join(chall_dir, DataImportTask.SEQUENCE_TSV)
            self.assertEqual(os.path.isfile(seq), True)

            nonpoly = os.path.join(chall_dir, DataImportTask.NONPOLYMER_TSV)
            self.assertEqual(os.path.isfile(nonpoly), True)

            fivehibdir = os.path.join(chall_dir, '5hib')
            fivehibtxt = os.path.join(fivehibdir, '5hib.txt')
            self.assertEqual(os.path.isfile(fivehibtxt), True)
            fivehibfas = os.path.join(fivehibdir, '5hib.fasta')
            self.assertEqual(os.path.isfile(fivehibfas), True)

            fivehicdir = os.path.join(chall_dir, '5hic')
            fivehictxt = os.path.join(fivehicdir, '5hic.txt')
            self.assertEqual(os.path.isfile(fivehictxt), True)
            fivehicfas = os.path.join(fivehicdir, '5hic.fasta')
            self.assertEqual(os.path.isfile(fivehicfas), True)

            # verify tarball is created
            tfile = chall.get_celpp_challenge_data_tar_file()
            self.assertEqual(os.path.isfile(tfile), True)

            foodir = os.path.join(temp_dir, 'foo')
            os.mkdir(foodir)

            tar = tarfile.open(tfile, 'r:*')
            tar.extractall(path=foodir)
            tar.close()
            name = chall.get_celpp_challenge_data_dir_name()
            cdir = os.path.join(foodir, name)
            readme = os.path.join(cdir, ChallengeDataTask.README_TXT_FILE)
            self.assertEqual(os.path.isfile(readme), True)

            final = os.path.join(foodir, name, 'final.log')
            self.assertEqual(os.path.isfile(final), False)

            e_con = os.path.join(foodir, name, 'error_container')
            self.assertEqual(os.path.isdir(e_con), False)

        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

__author__ = 'churas'

import os.path
import logging
import time
from d3r.celpp.task import D3RTask
from d3r.celpp import util

logger = logging.getLogger(__name__)


class MakeBlastDBTask(D3RTask):
    """Represents Blastable database

       Downloads pdb_seqres.txt from rcsb, gunzips it and
       runs NCBI makeblastdb to generate a blast database
       """

    PDB_SEQRES_TXT = "pdb_seqres.txt"
    PDB_SEQRES_TXT_GZ = (PDB_SEQRES_TXT + ".gz")

    # PDBID codes are currently 4 characters so plus 1 is 5
    LENGTH_OF_PDBID_PLUS_ONE = 5

    TASK_NAME = 'makeblastdb'

    def __init__(self, path, args):
        """Constructor

           Constructor sets name to makeblastdb and stage is set to 1
        """
        super(MakeBlastDBTask, self).__init__(path, args)
        self.set_name(MakeBlastDBTask.TASK_NAME)
        self.set_stage(1)
        self.set_status(D3RTask.UNKNOWN_STATUS)
        self._maxretries = 3
        self._retrysleep = 1

    def get_uploadable_files(self):
        """Returns list of files that can be uploaded to remote server

           List will contain these files in addition to stderr/stdout files
           if they are found on the filesystem

           pdb_seqres.txt.gz

           :returns: list of files that can be uploaded.
        """
        # get the stderr/stdout files
        file_list = super(MakeBlastDBTask, self).get_uploadable_files()

        pdbfile = self.get_pdb_seqres_txt_gz()

        if os.path.isfile(pdbfile):
            file_list.append(pdbfile)

        return file_list

    def get_pdb_seqres_txt_gz(self):
        """Returns path to pdb_seqres.txt.gz file
        :return: full path to MakeBlastDBTask.PDB_SEQRES_TXT_GZ file
        """
        return os.path.join(self.get_dir(), MakeBlastDBTask.PDB_SEQRES_TXT_GZ)

    def get_pdb_seqres_txt(self):
        """Returns path to pdb_seqres.txt file
        :return: full path to MakeBlastDBTask.PDB_SEQRES_TXT file
        """
        return os.path.join(self.get_dir(), MakeBlastDBTask.PDB_SEQRES_TXT)

    def _get_sequence_count_message(self):
        """Returns number of fasta sequences in get_pdb_seqres_txt() file

           Examines file set by get_pdb_seqres_txt and counts number of
           lines starting with > character which denotes the id of a fasta
           sequence entry
           :return: string of format: # sequence(s): XXX where is
                    a number upon success or the message Error unable to parse
                    file upon error
        """
        logger.debug('Examing ' + self.get_pdb_seqres_txt() +
                     ' to get sequence count')
        seq_count = 0
        try:
            f = open(self.get_pdb_seqres_txt(), 'r')
            for line in f:
                if line[0] == '>':
                    seq_count += 1
            f.close()
            return '# sequence(s): ' + str(seq_count)
        except:
            logger.exception('Caught exception trying to get sequence count')
        return '# sequence(s): Error unable to parse file'

    def get_set_of_pbdid_from_pdb_seqres_txt(self):
        """Parses MakeBlastDBTask.PDB_SEQRES_TXT to get set of PBDIDs

           This method parses `MakeBlastDBTask.PDB_SEQRES_TXT` fasta file to
           get a unique set of PBDIDs from the deflines (lines starting with >)

           Format of defline:

           >PBDID_# ###:##### length:###  ########

           The # above changes and PBDID is the PBDID code.

           This method will grab the PBDID which is 4 characters long,
           but for safety will parse on _
           character and return those PBDID as a set.  The PDBIDs will
           be made uppercase with upper() string function

           :returns: set of PBDIDS from sequence file where PDBIDs set
           to upper case via upper()
        """
        pdbid_set = set()
        start_time = int(time.time())
        logger.debug('Examing ' + self.get_pdb_seqres_txt() +
                     ' to get set of PDBIDs')

        if not os.path.isfile(self.get_pdb_seqres_txt()):
            logger.warning('')
            return set()
        try:
            f = open(self.get_pdb_seqres_txt(), 'r')
            for line in f:
                if line[0] == '>':
                    underscore_pos = line.find('_')
                    if underscore_pos != \
                            MakeBlastDBTask.LENGTH_OF_PDBID_PLUS_ONE:
                        logger.error('Skipping entry... defline in sequence '
                                     'file missing '
                                     'underscore at position 5: ' + line)
                        continue

                    pdbid_set.add(line[1:underscore_pos].upper())
            f.close()
            logger.debug('Found ' + str(len(pdbid_set)) +
                         ' PDBIDs from sequence.  Parsing took ' +
                         str(int(time.time()) - start_time) +
                         ' seconds.')
            return pdbid_set
        except:
            logger.exception('Caught exception trying to get set of PDBIDs')
            return set()
        return pdbid_set

    def can_run(self):
        """Determines if task can run

           Verifies this task does not already exist.
           If task does exist and does not have
           `D3RTask.COMPLETE_STATUS then self.get_error()
           is set with information about the issue
           :return: True if task can be run False otherwise
        """
        self._can_run = False
        self._error = None

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def run(self):
        """Generates pdb blast database

           First downloads pdb_seqres.txt.gz file from rcsb ftp,
           then gunzips file, and finally runs makeblastdb
           on file to generate blast database
        """
        super(MakeBlastDBTask, self).run()

        if self._can_run is False:
            logger.debug(self.get_dir_name() +
                         ' cannot run cause _can_run flag ' +
                         'is False')
            return

        try:
            logger.debug('pdbsequrl set to ' + self.get_args().pdbsequrl)
        except AttributeError:
            self.set_error('cannot download files cause pdbsequrl not set')
            self.end()
            return

        try:
            logger.debug('makeblastdb set to ' + self.get_args().makeblastdb)
        except AttributeError:
            self.set_error('cannot make blast database cause '
                           'makeblastdb not set')
            self.end()
            return

        download_path = self.get_pdb_seqres_txt_gz()
        url = self.get_args().pdbsequrl
        try:
            util.download_url_to_file(url,
                                      download_path,
                                      self._maxretries,
                                      self._retrysleep)

        except Exception:
            logger.exception('Caught Exception trying to download file: ' +
                             url)
            self.set_error('Unable to download file: ' + url)
            self.end()
            return

        try:
            # call util method to decompress file
            util.gunzip_file(download_path, self.get_pdb_seqres_txt())
        except IOError:
            logger.exception('Caught Exception trying to gunzip file: ' +
                             self.get_pdb_seqres_txt())
            self.set_error('Unable to uncompress file: ' +
                           self.get_pdb_seqres_txt())
            self.end()
            return

        # Run makeblastdb
        cmd_to_run = (self.get_args().makeblastdb + ' -in ' +
                      self.get_pdb_seqres_txt() +
                      ' -out ' + os.path.join(self.get_dir(), 'pdb_db') +
                      ' -dbtype prot')

        makeblastdb_name = os.path.basename(self.get_args().makeblastdb)

        self.run_external_command(makeblastdb_name, cmd_to_run, True)

        self.append_to_email_log('\n' + self._get_sequence_count_message() +
                                 '\n')
        # assess the result
        self.end()



# -*- coding: utf-8 -*-

__author__ = 'churas'

import logging
import re

logger = logging.getLogger(__name__)


class Participant(object):
    """Represents a an external participant in the challenge
    """

    def __init__(self, name, d3rusername, guid, email,
                 priority=0):
        """Constructor
        :param name: Full name of participant
        :param d3rusername: D3R username
        :param guid: unique id given to participant
        :param email: participant email address
        :param priority: int defining priority participants docked
                         results should be evaluated. default is 0
                         and higher values have higher priority
        :raises ValueError: if priority is set to a non int value
        """
        self._name = name
        self._d3rusername = d3rusername
        self._guid = guid
        self._email = email
        self._priority = int(priority)

    def get_name(self):
        """Gets name
        """
        return self._name

    def get_d3rusername(self):
        """Gets d3rusername
        """
        return self._d3rusername

    def get_guid(self):
        """Gets guid
        """
        return self._guid

    def get_email(self):
        """Gets email
        """
        return self._email

    def get_priority(self):
        """Gets priority for Participant used
           in determining order evaluations are run
        :returns: int denoting priority with higher
                  numbers meaning higher priority
        """
        return self._priority


class ParticipantDatabase(object):
    """Database object that contains a set of `Participants`
    """

    def __init__(self, participants):
        """Constructor
        :param participants: List of participants
        """
        self._participants = participants

    def get_participant_by_guid(self, guid,
                                exact_match=False):
        """Searches database for `Participant` with matching `guid`
        :param guid: guid or unique id for `Participant`
        :param exact_match: if set to True only guid matching
                            exactly with a `Participant` will be returned.
                            If False then the method will try for an exact
                            match, but if that fails the first `Participant`
                            that matches guid if _# is tripped of is used.
        :returns `Participant` with matching `guid` or None if not found
        """
        if guid is None:
            logger.warning('guid passed in was None')
            return None

        if self._participants is None:
            logger.warning('participants is None')
            return None

        for p in self._participants:
            if p.get_guid() == guid:
                return p

        if exact_match is True:
            logger.debug('No match found and exact_match is set '
                         'to True. returning None for guid: ' + guid)
            return None

        # TODO This wont work if the participant guids become a
        # TODO length other then 5 digits ie #####_XXXXX
        possible_guids = re.findall('([0-9]{5})_[0-9A-Za-z-]+$', guid)
        if len(possible_guids) is not 1:
            logger.debug('Stripped guid ' + guid +
                         ' did not yield single answer but this ' +
                         str(possible_guids))
            return None
        stripped_guid = possible_guids[0]
        if stripped_guid != guid:
            logger.debug('Stripped guid is different: ' + stripped_guid +
                         'performing search again')
            for p in self._participants:
                if p.get_guid() == stripped_guid:
                    return p

        return None

    def get_participants(self):
        """Gets participants as list
        :returns: List of `Participants`
        """
        return self._participants


class ParticipantDatabaseFromCSVFactory(object):
    """Factory class that generates `ParticipantDatabase` object from CSV file
    """

    def __init__(self, csvfile):
        """Constructor that takes CSV file `csvfile` to generate
        `ParticipantDatabase` object.  The format of the CSV file
        is as follows:

        name,d3rusername,guid,email
        bob smith,bsmith,8675309,bob@bob.com

        :param csvfile: Path to `Participant` CSV file
        """
        self._csvfile = csvfile

    def get_participant_database(self):
        """Parses CSV file set in constructor to create `ParticipantDatabase`
        :returns `ParticipantDatabase` from CSV file or None if there was an
        error
        """
        if self._csvfile is None:
            logger.warning('No csv file set')
            return None
        try:
            f = open(self._csvfile, 'rU')
            counter = 0
            plist = []
            for line in f:
                if counter is 0:
                    if line.startswith('name'):
                        logger.debug('Skipping header line')
                        counter = + 1
                        continue

                splitline = line.rstrip().split(',')
                split_len = len(splitline)
                if split_len is not 4 and split_len is not 5:
                    logger.warning('Problems splitting line ' + line +
                                   ' got ' + str(len(splitline)) +
                                   ' elements expecting 4 or 5 fields'
                                   ' name,username,guid,email or '
                                   ' name,username,guid,email,prority')
                    counter = + 1
                    continue
                priority = 0

                if split_len is 5:
                    try:
                        priority = int(splitline[4].rstrip())
                    except ValueError:
                        logger.error('Expected int for priority got: ' +
                                     splitline[4].rstrip())
                        pass

                plist.append(Participant(splitline[0].strip(),
                                         splitline[1].strip(),
                                         splitline[2].strip(),
                                         splitline[3].strip(),
                                         priority=priority))
                counter = + 1
            f.close()
            return ParticipantDatabase(plist)
        except Exception:
            logger.exception('Caught exception')
        return None



#!/usr/bin/env python

import commands
import os
import glob
import logging
import time
from openeye.oechem import *
import pickle
import numpy
import re

logger = logging.getLogger()
logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.INFO )


schrodingerScriptText = '''
import schrodinger.structutils.transform as ssut
import schrodinger.structure as ss
import numpy as np
import sys
if len(sys.argv) != 4:
    print "Usage: python temp.py inputmol outputmol rotationmatrix"
inFile = sys.argv[1]
outFile = sys.argv[2]
matrixFile = sys.argv[3]
rotMat = np.genfromtxt(matrixFile, skip_header=1)
#rotMat = np.concatenate((rotMat.T, np.zeros((1,4))))
rotMat = np.concatenate((rotMat, np.zeros((4,1))), axis=1)
rotMat[0,3] = rotMat[3,0]
rotMat[1,3] = rotMat[3,1]
rotMat[2,3] = rotMat[3,2]
#print rotMat
mol = ss.StructureReader(inFile).next()
ssut.transform_structure(mol, rotMat)
writer = ss.StructureWriter(outFile)
writer.append(mol)
writer.close()
'''


def merge_two_pdb (receptor, ligand, complex_pdb):
    complex_lines = []
    f1 = open(receptor, "r")
    protein_lines = f1.readlines()
    f1.close()
    for p_line in protein_lines:
        if p_line [:6] not in ["CONECT", "ENDMDL", "END   "]:
            complex_lines.append(p_line)
    complex_lines.append("TER   \n")
    f2 = open(ligand, "r")
    ligand_lines = f2.readlines()
    f2.close()
    for l_line in ligand_lines:
        if l_line [:6] not in ["REMARK", "MODEL ", "CONECT", "ENDMDL", "END   "]:
            complex_lines.append(l_line) 
    f3 = open(complex_pdb, "w")
    f3.writelines(complex_lines)
    f3.close()

def heavy_atom (mol):
#check the heavy atom numbers
    heavy_atom_numbers = 0
    for atom in mol.GetAtoms():
        if not atom.IsHydrogen():
            heavy_atom_numbers += 1
    return heavy_atom_numbers

def mcs(ref_mol, fit_mol, ref_ignore = [], fit_ignore = []):
#do the mcs search and return OEMatch object.
    #count heavy atoms 
    ref_mol_heavy_atoms = heavy_atom(ref_mol)
    fit_mol_heavy_atoms = heavy_atom(fit_mol)
    #ignore Hydrogen 
    OESuppressHydrogens(fit_mol)
    OESuppressHydrogens(ref_mol)
    #set atom and bond expression                                       
    atomexpr = OEExprOpts_AtomicNumber                                  
    bondexpr = 0                                                        
    #do the mcs search, using defined atom, bond expression options, and make sure using the small molecule as reference mol
    if not ref_mol_heavy_atoms > fit_mol_heavy_atoms:                   
        mcss = OEMCSSearch( ref_mol, atomexpr, bondexpr, True)          
    else:                                                               
        mcss = OEMCSSearch( fit_mol, atomexpr, bondexpr, True)          
    mcss.SetMCSFunc(OEMCSMaxAtomsCompleteCycles(1.5) )                  
    #create a new match object to store mcs search info.                
    new_match_list = []                                                 
    new_match_dic  = {}                                                 
    i = 0                                                               
    j = 0                                                               
    for match1 in mcss.Match(ref_mol):                                  
        i += 1                                                          
        #write out match1 molecule                                      
        mol1 = OEGraphMol()  
        OESubsetMol(mol1,match1, True)                                  
        match1_heavy_atom = heavy_atom(mol1)                            
        ofs1 = oemolostream("match1_%s.pdb"%i)                          
        OEWriteMolecule(ofs1, mol1)                                     
        ofs1.close()                                                    
        for match2 in mcss.Match(fit_mol):                              
            j += 1                                                      
            check_list = []                                             
            #write out match2 molecule                                  
            new_match = OEMatch()                                       
            mol2 = OEGraphMol()                                         
            OESubsetMol(mol2,match2, True)                              
            match2_heavy_atom = heavy_atom(mol2)
            ofs2 = oemolostream("match2_%s.pdb"%j)                      
            OEWriteMolecule(ofs2, mol2)                                 
            ofs2.close()                                                
            if match1_heavy_atom != match2_heavy_atom:                  
                print "WARNING: match1_%s.pdb and match2_%s.pdb have different heavy atoms, need to check..."%(i,j)
            for mp1, mp2 in zip(match1.GetAtoms(), match2.GetAtoms()):  
                ref_name = mp1.target.GetName().strip()                 
                fit_name = mp2.target.GetName().strip()                 
                if (ref_name not in ref_ignore) and (fit_name not in fit_ignore ) :   
                    new_match.AddPair (mp1.target, mp2.target)          
                else:                                                   
                    print "             Deleted atom pair", mp1.target.GetName(), mp2.target.GetName()
            new_match_list.append(new_match)                            
            new_match_dic[new_match] = (["match1_%s_vs_match2_%s"%(i,j), check_list ])
    return new_match_dic 

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

def add_symbol (pdb_file):
    # check the file format, if it is pdb file then do the add.
    if os.path.splitext(pdb_file)[-1] == ".pdb":
        f = open(pdb_file, "r")
        lines = f.readlines()
        new_lines = []
        for line in lines:
            #lines with atom informations  
            if "ATOM" or "HETATM" in line.split()[0]:
                if is_number(line.split()[-1]) and is_number(line.split()[-2]):
                    #ensure the end has no atom symbol, which means the end is number. Also ensure the end is not charge. 
                    atom_name = line.split()[2]
                    #remove numbers in atom name.                       
                    atom_symbol = ''.join(i for i in atom_name if not i.isdigit())     
                    #add whitespace because of the requirment of the pdb format       
                    lenth_symbol = "          " + "%s"%atom_symbol      
                    new_line = line.split("\n")[0].strip()+ lenth_symbol + "\n"       
                    new_lines.append(new_line)                          
                else:                                                   
                    new_lines.append(line)                              
            else:                                                       
                new_lines.append(line)                                  
        file_name, file_extension = os.path.splitext(pdb_file)          
        out_file = file_name + "_new_atom_symbol" + file_extension      
        f2 = open(out_file,"w")                                         
        f2.writelines(new_lines)
        return out_file                                                 
    else:                                                               
        #if the format is not pdb then return original                  
        return pdb_file

def main_rmsd(ref_struc, fit_struc):
    reffs = oemolistream()
    reffs.open(add_symbol(ref_struc))
    fitfs = oemolistream() 
    fitfs.open(add_symbol(fit_struc))
    refmol = OEGraphMol()
    OEReadMolecule(reffs, refmol)
    for fitmol in fitfs.GetOEGraphMols():
        rmsd  = OERMSD(refmol, fitmol)
        rot  = OEDoubleArray(9)
        trans = OEDoubleArray(3)
        automorph = True
        heavyOnly = True
        overlay = True
        rmsd  = OERMSD(refmol, fitmol, automorph, heavyOnly, overlay, rot, trans)
        ss = mcs(refmol, fitmol,)
        mcss_rmsd_list = []
        match_info = []
        for mcss in ss.keys():
            #mcss_rmsd = OERMSD(refmol, fitmol, mcss, overlay, rot, trans)
            mcss_rmsd = OERMSD(refmol, fitmol, mcss)
            mcss_rmsd_list.append(mcss_rmsd)
            match_info.append(ss[mcss])
        for filename in glob.glob('*_new_atom_symbol.pdb'):             
            os.remove(filename) 
        SMCSS_number_index = mcss_rmsd_list.index(min(mcss_rmsd_list))
        biggest_number_index = mcss_rmsd_list.index(max(mcss_rmsd_list))
    return min(mcss_rmsd_list)


def old_structure_align(template_pro, target_pro):
    commands.getoutput("$SCHRODINGER/utilities/structalign %s %s"%(template_pro, target_pro))
    out_rot_target_pro = "rot-" + target_pro
    if os.path.isfile(out_rot_target_pro):
        return out_rot_target_pro
    else:
        logging.info("Could not algin %s onto %s"%(target_pro,template_pro))

def structure_align(prefix, actual_xtal_pdb, receptor_in, ligand_in):
    receptor_mae = 'rot_%s_receptor1.mae' %(prefix)
    receptor_out = 'rot_%s' %(receptor_in)
    ligand_out = 'rot_%s' %(ligand_in)
    #matrixFile = receptor_in.replace('.pdb','.rot')
    matrixFile = 'temp_receptor.rot'
    with open('temp.py','wb') as of:
        of.write(schrodingerScriptText)
    
    
    ## On the old version of maestro/structalign that's on nif1, we can't get the rotation matrix by aligning two pdbs. Until it gets updated, I'll convert a copy to mae and get the rotation matrix by aligning that.
    
    cmd0 = '$SCHRODINGER/utilities/structconvert -ipdb %s -omae temp_xtal.mae >& %s-0-convertXtalPdbToMae' %(actual_xtal_pdb, prefix)
    commands.getoutput(cmd0)
    cmd0point5 = '$SCHRODINGER/utilities/structconvert -ipdb %s -omae temp_receptor.mae >& %s-0.5-convertCanPdbToMae' %(receptor_in, prefix)
    commands.getoutput(cmd0point5)
    
    ## NOTE! the default rotation matrix output by structalign only has 3 decimal places. This might make a difference. 
    ## To fix it, make a copy of $SCHRODINGER/mmshare-v<whatever>/bin/Linux-x86_64/structalign_utility.py in which the %.3f's in the mat.write lines are replaced with %.5f's.
    #cmd1 = '$SCHRODINGER/utilities/structalign -matrix %s %s >& %s_structAlignOut' %(actual_xtal_pdb, receptor_in, prefix)
    cmd1 = '$SCHRODINGER/utilities/structalign -matrix temp_xtal.mae temp_receptor.mae >& %s-1-alignMatrixGenOut' %(prefix)
    commands.getoutput(cmd1)
    
    cmd2 = '$SCHRODINGER/utilities/python temp.py %s %s %s >& %s-2-receptorAlignOut' %(receptor_in, receptor_mae, matrixFile, prefix)
    commands.getoutput(cmd2)
    
    cmd3 = '$SCHRODINGER/utilities/python temp.py %s %s %s >& %s-3-ligandAlignOut' %(ligand_in, ligand_out, matrixFile, prefix)
    commands.getoutput(cmd3)
    
    cmd4 = '$SCHRODINGER/utilities/structconvert -imae %s -opdb %s >& %s-4-structConvertOut' %(receptor_mae, receptor_out, prefix)
    commands.getoutput(cmd4)
    
    return receptor_out, ligand_out

def make_complex_pdb(receptor_pdb, ligand_mol, complex_pdb):
    ## First, convert the ligand to pdb
    ligand_pdb = ligand_mol.replace('.mol','_lig.pdb')
    commands.getoutput('babel -imol %s -opdb %s' %(ligand_mol, ligand_pdb))
    ## Now combine the ligand and receptor pdbs
    commands.getoutput('babel --join -ipdb %s -ipdb %s -opdb %s' %(receptor_pdb, ligand_pdb, complex_pdb))
    return ligand_pdb, complex_pdb
 

def layout_result (pickle_file, out_file, outformat = "txt"):
    data = []
    p_file = open(pickle_file,"r")
    score_dic = pickle.load(p_file)
    p_file.close()
    if outformat == "csv":
        data = ["%-20s%-10s, %-10s, %-10s, %-10s, %-10s \n"%("Target_PDBID,", "LMCSS", "SMCSS", "hiResApo", "hiResHolo", "hiTanimoto")]
    elif outformat =="txt":
        data = ["%-20s%-10s%-10s%-10s%-10s%-10s \n"%("Target_PDBID", "LMCSS", "SMCSS", "hiResApo", "hiResHolo", "hiTanimoto")]
    #add hiTanimoto 0909 sliu
    all_pro_type = ["LMCSS", "SMCSS", "hiResApo", "hiResHolo", "hiTanimoto"]
    LMCSS_list =  []
    SMCSS_list = []
    hiResApo_list = []
    hiResHolo_list = []
    hiTanimoto_list = []
    abnormal_dic = {}
    total_pdb = 0
    for pdbid in score_dic:
        for pro_type in all_pro_type:
            if pro_type in score_dic[pdbid]:
                if pro_type == "LMCSS":
                    LMCSS_list.append(score_dic[pdbid][pro_type])
                    if float(score_dic[pdbid][pro_type]) > 8.0:
                        abnormal_dic[pdbid] = score_dic[pdbid][pro_type]
                if pro_type == "SMCSS":
                    SMCSS_list.append(score_dic[pdbid][pro_type])
                if pro_type == "hiResApo":
                    hiResApo_list.append(score_dic[pdbid][pro_type]) 
                if pro_type == "hiResHolo":
                    hiResHolo_list.append(score_dic[pdbid][pro_type])
                if pro_type == "hiTanimoto":
                    hiTanimoto_list.append(score_dic[pdbid][pro_type])
        total_pdb += 1
    if outformat == "csv":
        data.append("%-20s%-10s, %-10s, %-10s, %-10s, %-10s, \n"%("Number_of_cases,", len(LMCSS_list), len(SMCSS_list), len(hiResApo_list), len(hiResHolo_list), len(hiTanimoto_list)))
    elif outformat == "txt":
        data.append("%-20s%-10s%-10s%-10s%-10s%-10s\n"%("Number_of_cases", len(LMCSS_list), len(SMCSS_list), len(hiResApo_list), len(hiResHolo_list), len(hiTanimoto_list)))
    
    whole_list = (LMCSS_list, SMCSS_list, hiResApo_list, hiResHolo_list, hiTanimoto_list)
    if outformat == "csv":
        average_line, max_line, min_line = ("%-20s"%"Average,", "%-20s"%"Maximum,", "%-20s"%"Minimum,")
    elif outformat == "txt":
        average_line, max_line, min_line = ("%-20s"%"Average", "%-20s"%"Maximum", "%-20s"%"Minimum")
    for index, this_list in enumerate (whole_list):
        if len(this_list) == 0:
            if outformat == "csv":
                average_line += "%-10s, "%(" ")
                min_line += "%-10s, "%(" ")
                max_line += "%-10s, "%(" ")
            elif outformat == "txt":
                average_line += "%-10s"%(" ")
                min_line += "%-10s"%(" ")
                max_line += "%-10s"%(" ")
        else:
            if outformat == "csv":
                average_line += "%-10.3f, "%(numpy.average(this_list))
                min_line += "%-10.3f, "%(min(this_list))
                max_line += "%-10.3f, "%(max(this_list))
            if outformat == "txt":
                average_line += "%-10.3f"%(numpy.average(this_list))
                min_line += "%-10.3f"%(min(this_list))
                max_line += "%-10.3f"%(max(this_list))
    average_line += "\n" 
    max_line += "\n"
    min_line += "\n"
    data.append(average_line)
    data.append(min_line)
    data.append(max_line)
       
    #add main score lines 
    for pdbid in score_dic:
        new_line_score = ""
        valid_line = False
        for pro_type in all_pro_type:
            if pro_type in score_dic[pdbid]:
                if outformat == "csv":
                    new_line_score += "%-10.3f, "%score_dic[pdbid][pro_type] 
                if outformat == "txt":
                    new_line_score += "%-10.3f"%score_dic[pdbid][pro_type] 
                valid_line = True
            else:
                if outformat == "csv":
                    new_line_score += "%-10s, "%(" ")
                if outformat == "txt":
                    new_line_score += "%-10s"%(" ")
        if valid_line:
            if outformat == "csv":
                pdbid_full = "%s,"%pdbid
            if outformat == "txt":
                pdbid_full = "%s"%pdbid
            new_line = "%-20s"%(pdbid_full)
            new_line += new_line_score
            new_line += "\n" 
            data.append(new_line)
    #data.append("=====Total number of target protein: %s =====\n"%total_pdb)
    #append total number of valid pdb for each type 
    #data.append("%-20s%-10s, %-10s, %-10s, %-10s, %-10s, \n"%("Valid_cases,", len(LMCSS_list), len(SMCSS_list), len(hiResApo_list), len(hiResHolo_list), len(hiTanimoto_list)))
    #data.append("%-20s%-10.3f, %-10.3f, %-10.3f, %-10.3f, %-10.3f, \n"%("Average,", numpy.average(LMCSS_list), numpy.average(SMCSS_list), numpy.average(hiResApo_list), numpy.average(hiResHolo_list), numpy.average(hiTanimoto_list)))
    #data.append("=====Abnormal RMSDs (LMCSS cases where the RMSD > 8.0)=====\n")
    #for abnormal_id in abnormal_dic:
    #    abnormal_id_full = "\"%s\","%abnormal_id
    #    data.append("%-20s%-10.3f, \n"%(abnormal_id_full, abnormal_dic[abnormal_id]))
    out_txt = open(out_file, "w")
    out_txt.writelines(data)
    out_txt.close()  
     

def main_score (dock_dir, pdb_protein_path, evaluate_dir, update= True):
    pickle_file_name = "RMSD.pickle"
    pickle_out = open (pickle_file_name, "w")
    score_dic = {}
    os.chdir(evaluate_dir)
    current_dir = os.getcwd()
    #target_dirs = []
    #all_pdbids = next(os.walk(dock_dir))[1]    
    ##all_pdbids = ['4xm7']
    #for all_pdb_path in os.walk(dock_dir):
    #    if all_pdb_path[0] != dock_dir:
    #        
    #        if all_pdb_path[0].split("/")[-1] in all_pdbids:
    #            #make sure that we didn't visit the deep folders other than the pdbid ones
    #            target_dirs.append(all_pdb_path[0])
    
    ## os.walk will return a tuple of (directory absolute path, subdirectories, files)

    target_dirs = list(os.walk(dock_dir))[0][1]

    
    for target_dir in target_dirs:
        all_docked_structures = []
        valid_target = False # this flag indicates whether we have at least one valid docked structure for scoring
        commands.getoutput("cp -r %s/%s %s"%(dock_dir,target_dir, evaluate_dir))
        target_name = os.path.basename(target_dir)
        os.chdir(target_name)
        ## Ensure the target is in score_dic
        if target_name not in score_dic:
            score_dic[target_name] = {}
            
        #now we are in folders named by pdbid
        current_dir_layer_2 = os.getcwd()
        ##################
        #Do the scoring here
        #1, get all the docked structures and crystal structure
        potential_mols = glob.glob('*docked.mol' )
        ## Go copy in all the submitted poses
        for potential_mol in potential_mols:
            
            ## First, ensure that this ligand has a corresponding receptor
            potential_receptor = potential_mol.replace('.mol','.pdb')
            if not(os.path.isfile(potential_receptor)):
                logging.info('There is no receptor corresponding to docked molecule %s. Skipping scoring this pose.' %(potential_mol))
                continue
            
            ## If it does, set up the directory for scoring 
            if valid_target == False:
                valid_target = True
                os.mkdir("score")
                
            ## don't copy in the original pdb and mol files - Just convert them to mae
            #complex_mae = os.path.basename(potential_mol).replace('.mol','_complex.mae')
            #commands.getoutput('$SCHRODINGER/utilities/structcat -imol score/%s -ipdb score/%s -omae score/%s' %(potential_mol, potential_receptor, complex_mae))
            #all_docked_structures.append(complex_mae)
            
            ## copy in the structures intended for scoring
            commands.getoutput("cp %s score" %potential_mol)
            commands.getoutput("cp %s score" %potential_receptor)
            
            ## Combine them into a single structure for RSR calculation
            ## The score subdirectory is flat, so we need to drop the target directory structure attached to potential_mol
            local_potential_mol = os.path.basename(potential_mol)
            local_potential_receptor = os.path.basename(potential_receptor)
            #intermediate_lig_pdb = local_potential_mol.replace('.mol','_ligand.pdb')
            ## The complex mae will be called "<method>-<target>_<candidate>_docked_complex.pdb"
            #complex_pdb = local_potential_mol.replace('.mol','_complex.pdb')
            #commands.getoutput('babel -imol score/%s -opdb score/%s' %(potential_mol, intermediate_lig_pdb))
            #commands.getoutput('babel --join -ipdb score/%s -ipdb score/%s -opdb score/%s' %(potential_receptor, intermediate_lig_pdb, complex_pdb))
            
            
            ## Package the ligand, receptor, and complex file up for the next step
            all_docked_structures.append((local_potential_mol, local_potential_receptor))
        
        ## Check whether it makes sense to continue scoring
        if not(valid_target):
            #no docked structure available for this case, skip this one
            logging.info("There are no valid docked structures for this PDBID: %s"%target_name)
            os.chdir(current_dir)
            continue
        
        ## Extract the crystal (local pdb database is organized by middle 2 characters of pdb codes)
        crystal_id = target_name
        crystal_pdb_folder_name = crystal_id[1:3]    
        crystal_ent_file = "pdb" + crystal_id + ".ent"
        crystal_pdbloc = os.path.join(pdb_protein_path,crystal_pdb_folder_name, crystal_ent_file )
        ## Ensure that the xtal entry for this pdb code exists
        if not os.path.isfile(crystal_pdbloc):
            logging.info("Unable to find the ent file associate with the crystal pdb: %s at location %s"%(crystal_id, crystal_pdbloc))
            os.chdir(current_dir)
            continue
        else:
            commands.getoutput("cp %s score/crystal.pdb"%(crystal_pdbloc))
            
        ## Finish step 1 
        ## Move into score dir 
        os.chdir("score")    
        logging.info("=============Start working in this case:%s=========="%target_name)
        #2, split the maegz file into pdb files and combine receptor with ligand
        #align the complex onto the crystal structure and split into ligands with receptor
        
        #all_docked_structures = glob.glob("*_docked.mol") 
        ## remove all files start with rot-
        #previous_files = glob.glob("rot-*_docked.mol")
        #for item in previous_files:
        #    all_docked_structures.remove (item)
        
        ## Split the xtal ligands from the xtal stucture
        commands.getoutput("$SCHRODINGER/run split_structure.py -many_files -m ligand crystal.pdb crystal.pdb")
        if not os.path.isfile("crystal_ligand1.pdb"):
            logging.info("Crystal structure is not splittable. Skipping")
            os.chdir(current_dir_layer_2)
            os.chdir(current_dir)
            continue    
        else:
            # Get the filenames of all the crystal ligands
            crystal_ligand_list = glob.glob("crystal_ligand*.pdb")
        
        ## Remember that all_docked_structures is a list of tuples: (potential_mol, potential_receptor, complex_pdb))
        for docked_lig_mol, docked_receptor in all_docked_structures:

            ## Get the method type, target, and candidate info from the filename
            # for example, this will parse 'hiResApo-5hib_2eb2_docked.mol' into [('hiResApo', '5hib', '2eb2')]
            parsed_name = re.findall('([a-zA-Z0-9]+)-([a-zA-Z0-9]+)_([a-zA-Z0-9]+)_docked.mol',docked_lig_mol)
            if len(parsed_name) != 1:
                logging.info('Failed to parse docked structure name "%s". Parsing yielded %r' %(docked_lig_mol, parsed_name))
                continue
            docked_structure_type = parsed_name[0][0]
            docked_structure_target = parsed_name[0][1]
            docked_structure_candidate = parsed_name[0][2]


            ## Align the submission to the crystal structure
            file_prefix = "%s-%s_%s_docked" %(docked_structure_type,
                                              docked_structure_target,
                                              docked_structure_candidate)             
            aln_complex_pdb = '%s_complex.pdb' %(file_prefix)
                
            ## Loop over permutations of crystal and docked chains
            commands.getoutput("$SCHRODINGER/run split_structure.py -many_files -m chain crystal_receptor1.pdb crystal_chains.pdb")
            crystal_chain_list = glob.glob('crystal_chains_chain?.pdb')
            commands.getoutput("$SCHRODINGER/run split_structure.py -many_files -m molecule %s docked_molecules.pdb" %(docked_receptor))
            docked_chain_list = glob.glob('docked_molecules_mol*.pdb')

            for crystal_chain_pdb in crystal_chain_list:
                for docked_chain in docked_chain_list:
                    aln_recep, aln_lig = structure_align(file_prefix, crystal_chain_pdb, docked_chain, docked_lig_mol)
                    aln_lig_pdb, aln_complex_pdb = make_complex_pdb(aln_recep, aln_lig, aln_complex_pdb)
                    #docked_lig_pdb = docked_lig_mol.replace('.mol','.pdb')


                    ## Now compare the RMSDs of every ligand to those in the crystal
                    ## We need the ligand as a pdb
                    try:
                        rmsd_list = []
                        for crystal_ligand in crystal_ligand_list:
                            rmsd = main_rmsd(crystal_ligand, aln_lig_pdb)
                            logging.info( "RMSD for the first ligand: %s comparing with crystal ligand :%s, is : %s"%(aln_lig_pdb, crystal_ligand, rmsd))
                            rmsd_list.append(rmsd)
                        if docked_structure_type not in score_dic[target_name]:
                            score_dic[target_name][docked_structure_type] = []
                        score_dic[target_name][docked_structure_type] += rmsd_list
                    except:
                        logging.info("RMSD cannot be calculated for the ligand: %s"%(aln_lig_pdb))
            score_dic[target_name][docked_structure_type] = min(score_dic[target_name][docked_structure_type])



        #Finish scoring, come back to the folder with pdbid
        os.chdir(current_dir_layer_2)
        # Finish scoring for this pdbid, come back to the main folder
        os.chdir(current_dir)
        ##################
    pickle.dump(score_dic, pickle_out)
    pickle_out.close()
    return pickle_file_name

if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-d", "--dockdir", metavar="PATH",
                      help="PATH where we could find the docking stage output")

    parser.add_argument("-o", "--outdir", metavar="PATH",
                      help="PATH where we will run the evaluate stage")

    parser.add_argument("-p", "--pdbdb", metavar="PATH",
                      help="PDB DATABANK which we will "
                           "get the crystal structure")
    logger = logging.getLogger()
    logging.basicConfig(format='%(asctime)s: %(message)s',
                        datefmt='%m/%d/%y %I:%M:%S', filename='final.log',
                        filemode='w', level=logging.INFO)
    opt = parser.parse_args()
    dockDir = opt.dockdir
    evaluateDir = opt.outdir
    pdbloc = opt.pdbdb
    # running under this dir
    running_dir = os.getcwd()
    pickle_result = main_score(dockDir, pdbloc, evaluateDir, )
    pickle_loc = os.path.join(running_dir, "RMSD.pickle")
    txt_result = layout_result(pickle_loc, "RMSD.txt", outformat = "txt")
    txt_result = layout_result(pickle_loc, "RMSD.csv", outformat = "csv")
    # move the final log file to the result dir
    log_file_path = os.path.join(running_dir, 'final.log')
    commands.getoutput("mv %s %s" % (log_file_path, evaluateDir))
    pickle_file_path = os.path.join(running_dir, 'RMSD.pickle')
    commands.getoutput("mv %s %s" % (pickle_file_path, evaluateDir))



__author__ = 'churas'

import os
import logging

from d3r.celpp.task import SmtpEmailer
from d3r.celpp.task import Attachment
from d3r.celpp import util
from d3r.celpp.task import D3RTask
from d3r.celpp.evaluation import EvaluationTask
from d3r.celpp.evaluation import EvaluationTaskFactory
from d3r.celpp.challengedata import ChallengeDataTask

logger = logging.getLogger(__name__)


class PathNotDirectoryError(Exception):
    """Path is not a directory Error
    """
    pass


class PostEvaluationEmailer(object):
    """Sends evaluation email
    """
    def __init__(self, to_list, reply_to_address,
                 smtp='localhost', smtpport=25):
        self._alt_smtp_emailer = None
        self._msg_log = None
        self._to_list = to_list
        self._reply_to_address = reply_to_address
        self._smtp = smtp
        self._smtpport = smtpport

    def set_alternate_smtp_emailer(self, emailer):
        """Sets alternate smtp emailer
        """
        self._alt_smtp_emailer = emailer

    def _get_smtp_emailer(self):
        """Gets `SmtpEmailer` object or alternate
        """
        if self._alt_smtp_emailer is not None:
            return self._alt_smtp_emailer
        return SmtpEmailer(smtp_host=self._smtp,
                           port=self._smtpport)

    def _append_to_message_log(self, msg):
        if self._msg_log is None:
            self._msg_log = str(msg)
            return
        self._msg_log += str(msg)

    def get_message_log(self):
        """Gets the error log for this object
        :returns: Error log as string
        """
        return self._msg_log

    def _get_reply_to_address(self, from_address):
        """Gets reply to address
        :returns: reply to email address
        """
        if self._reply_to_address is None:
            return from_address
        return self._reply_to_address

    def _generate_post_evaluation_email_body(self, petask):
        """Creates body of email and subject
        :returns subject,body: as strings
        """
        weekno = util.get_celpp_week_number_from_path(petask.get_path())
        year = util.get_celpp_year_from_path(petask.get_path())

        msg = ('Dear CELPP Admin,\n\nHere is the post evaluation  ' +
               'summary reports for CELPP week ' + str(weekno) + '\n\n')

        msg += petask.get_postevaluation_summary()

        msg += '\n\nSincerely,\n\nCELPP Automation'

        subject = (D3RTask.SUBJECT_LINE_PREFIX + str(year) + ' ' +
                   'week ' + str(weekno) + ' post evaluation summary report')
        return subject, msg

    def send_postevaluation_email(self, petask):
        """Sends evaluation email appending issues to message log
        """

        # clear message log
        self._msg_log = None
        if petask is None:
            logger.error('Task passed in is None')
            self._append_to_message_log('\nTask passed in is None\n')
            return

        try:
            emailer = self._get_smtp_emailer()

            if self._to_list is None:
                logger.debug('No email addresses in to list')
                self._append_to_message_log('\nNo email addresses ' +
                                            'in to list\n')
                return

            subject, msg = self._generate_post_evaluation_email_body(petask)

            from_addr = emailer.generate_from_address_using_login_and_host()

            reply_to = self._get_reply_to_address(from_addr)

            attach_list = None
            for csv_file in petask.get_all_csv_files_in_dir():
                if attach_list is None:
                    attach_list = []
                attach_list.append(Attachment(csv_file,
                                              os.path.basename(csv_file)))

            emailer.send_email(from_addr, self._to_list, subject, msg,
                               reply_to=reply_to,
                               attachments=attach_list)

            self._append_to_message_log('\nSent post evaluation email to: ' +
                                        ", ".join(self._to_list) + '\n')
        except Exception as e:
            logger.exception('Caught exception')
            self._append_to_message_log('\nCaught exception trying to email '
                                        ': ' +
                                        ', '.join(self._to_list) + ' : ' +
                                        str(e) + '\n')


class PostEvaluationTask(D3RTask):
    """Generates summary of all evaluations run
    """
    RMSD_TXT = 'RMSD.txt'
    RMSD_PICKLE = 'RMSD.pickle'
    TASK_NAME = 'postevaluation'
    FINAL_LOG = 'final.log'
    CSV_SUFFIX = '.csv'
    SUMMARY_TXT = 'summary.txt'
    EVALUATIONDIR_ARG = '--evaluationdir'

    def __init__(self, path, args):
        super(PostEvaluationTask, self).__init__(path, args)
        self.set_name(PostEvaluationTask.TASK_NAME)
        et = EvaluationTask('/foo', '', None, None)
        self._eval_task_prefix_str = et.get_dir_name()
        self._eval_task_suffix_str = ('' +
                                      EvaluationTask.EXT_SUBMISSION_SUFFIX +
                                      '.' +
                                      EvaluationTaskFactory.SCORING_SUFFIX +
                                      '$|.' +
                                      EvaluationTaskFactory.SCORING_SUFFIX +
                                      '$')
        self.set_stage(et.get_stage() + 1)
        self._emailer = None
        self.set_status(D3RTask.UNKNOWN_STATUS)

    def set_evaluation_emailer(self, emailer):
        self._emailer = emailer

    def get_all_evaluation_tasks(self):
        """Look for all  evaluation tasks
        """
        task_list = []
        prefix_len = len(self._eval_task_prefix_str)
        base_dir = self.get_path()
        for entry in os.listdir(base_dir):
            if not entry.endswith('.' + EvaluationTaskFactory.SCORING_SUFFIX):
                continue
            full_path = os.path.join(base_dir, entry)
            if not os.path.isdir(full_path):
                logger.info('Suffix matched, but is not directory: ' +
                            full_path)
                self.append_to_email_log('Just a note, found a task ' +
                                         'with valid name, but it is not '
                                         'a directory ' + full_path)
                continue
            task = EvaluationTask(base_dir, entry[prefix_len:], None,
                                  self.get_args())
            task.update_status_from_filesystem()
            if task.get_status() != D3RTask.COMPLETE_STATUS:
                logger.info('fyi ' + task.get_name() + ' task ' +
                            ' has a status of ' + task.get_status())
            task_list.append(task)
        return task_list

    def _get_evaluationdir_args(self):
        """Generates a string containing all the evaluation
           tasks to pass to post_evaluation.py script
        :returns: string of format --evaluationdir <path1>
                  --evaluationdir <path2>
        """
        eval_args = ''
        for task in self.get_all_evaluation_tasks():
            logger.debug('Adding ' + task.get_name() + ' to arglist')
            eval_args += (' ' + PostEvaluationTask.EVALUATIONDIR_ARG + ' ' +
                          task.get_dir())
        return eval_args

    def get_all_csv_files_in_dir(self):
        """Gets all the files ending with CSV_SUFFIX
           in task directory
        """
        csv_list = []
        out_dir = self.get_dir()
        for entry in os.listdir(out_dir):
            if entry.endswith(PostEvaluationTask.CSV_SUFFIX):
                full_path = os.path.join(out_dir, entry)
                if os.path.isfile(full_path):
                    csv_list.append(full_path)
                else:
                    logger.warning(full_path +
                                   ' is a directory which is weird')
        return csv_list

    def get_summary_txt(self):
        """Gets summary.txt file path
        """
        return os.path.join(self.get_dir(), PostEvaluationTask.SUMMARY_TXT)

    def get_postevaluation_summary(self):
        """Summarizes post evaluation results into a string

        """
        summary_file = self.get_summary_txt()
        start_line = '\n\n'
        if not os.path.isfile(summary_file):
            return start_line + 'No ' + summary_file + ' file found\n'

        try:
            f = open(summary_file, 'r')
            summary = f.read()
            f.close()
            return start_line + summary + '\n'
        except Exception as e:
            logger.exception('Caught exception')
            return (start_line + 'Unable to generate postevaluation '
                                 'summary (' + str(e) + ')\n')

    def get_uploadable_files(self):
        """Returns list of files that can be uploaded to remote server

           List will contain these files if found on the file system
           plus stderr/stdout files

           *.csv
           final.log
           :returns: list of files that can be uploaded
        """
        # get the stderr/stdout files
        file_list = super(PostEvaluationTask, self).get_uploadable_files()
        out_dir = self.get_dir()
        try:
            final_log = os.path.join(out_dir, PostEvaluationTask.FINAL_LOG)
            if os.path.isfile(final_log):
                file_list.append(final_log)

            if os.path.isfile(self.get_summary_txt()):
                file_list.append(self.get_summary_txt())

            csv_files = self.get_all_csv_files_in_dir()
            logger.debug('Appending ' + str(len(csv_files)) + ' csv files')
            file_list.extend(csv_files)
        except OSError:
            logger.exception('Caught exception looking for csv files')
        return file_list

    def can_run(self):
        """Determines if task can actually run

           The method verifies this task does
           not already exist.  If above is not true then self.set_error()
           is set with information about the issue
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None
        # check this task is not complete and does not exist

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def run(self):
        """Runs PostEvaluationTask

           Method requires can_run() to be called before hand with
           successful outcome
           Otherwise method invokes D3RTask.start then this method
           creates a directory and invokes evaluation script.  Upon
           completion results are analyzed and success or error status
           is set appropriately and D3RTask.end is invoked
           """
        super(PostEvaluationTask, self).run()

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return

        try:
            logger.debug('postevaluation set to ' +
                         self.get_args().postevaluation)
        except AttributeError:
            self.set_error('postevaluation not set')
            self.end()
            return

        #
        # --evaluationdir <path to stage.X.evaluation>
        # --evaluationdir <path to another etc...>
        # --challengedir <path to challenge dir ie
        #                 stage.4.challengedata/celpp_week10_2017>
        #
        chall = ChallengeDataTask(self.get_path(), self.get_args())
        chall_dname = chall.get_celpp_challenge_data_dir_name()

        evaldir_args = self._get_evaluationdir_args()
        if evaldir_args is not '':
            cmd_to_run = (self.get_args().postevaluation + ' ' +
                          self.get_dir() + ' ' +
                          evaldir_args +
                          ' --challengedir ' +
                          os.path.join(chall.get_dir(),
                                       chall_dname) +
                          ' --stageprefix ' + self._eval_task_prefix_str +
                          ' --evaluationsuffix ' + self._eval_task_suffix_str)
            peval_name = os.path.basename(self.get_args().postevaluation)

            self.run_external_command(peval_name, cmd_to_run,
                                      True)
        else:
            self.set_error('Did not find any Evaluation tasks to summarize')

        # attempt to send postevaluation email
        try:
            self._emailer.send_postevaluation_email(self)
            self.append_to_email_log(self._emailer.get_message_log())
        except Exception as e:
            logger.exception('Caught exception trying to send '
                             'postevaluation email')
            self.append_to_email_log('Caught exception trying to send '
                                     'evaluation email ' + str(e) + '\n')

        # assess the result
        self.end()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'churas'

import unittest
import tempfile
import os.path

"""
test_blastnfilter
--------------------------------

Tests for `blastnfilter` module.
"""

import shutil
import os
import stat
from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.blastnfilter import BlastNFilterTask
from d3r.celpp.blastnfilter import BlastNFilterSummary
from d3r.celpp.dataimport import DataImportTask
from d3r.celpp.makeblastdb import MakeBlastDBTask
from tests.celpp import test_task


class TestBlastNFilterTask(unittest.TestCase):
    def setUp(self):
        pass

    def test_update_status_from_filesystem(self):
        params = D3RParameters()
        task = BlastNFilterTask(None, params)
        test_task.try_update_status_from_filesystem(self, task)

    def test_constructor(self):
        params = D3RParameters()
        blasttask = BlastNFilterTask('ha', params)
        self.assertEqual(blasttask.get_name(), 'blastnfilter')
        self.assertEqual(blasttask.get_path(), 'ha')
        self.assertEqual(blasttask.get_stage(), 3)
        self.assertEqual(blasttask.get_status(), D3RTask.UNKNOWN_STATUS)
        self.assertEqual(blasttask.get_error(), None)

    def test_get_txt_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            blast_task = BlastNFilterTask(temp_dir, params)

            # try missing directory
            self.assertEquals(len(blast_task.get_txt_files()), 0)

            blast_task.create_dir()

            # try empty directory
            self.assertEquals(len(blast_task.get_txt_files()), 0)

            # try where only summary.txt exists
            summary_file = os.path.join(blast_task.get_dir(), 'summary.txt')
            open(summary_file, 'a').close()
            self.assertEquals(len(blast_task.get_txt_files()), 0)

            # try non txt files only
            os.remove(summary_file)
            open(os.path.join(blast_task.get_dir(), 'foo.csv'), 'a').close()
            self.assertEquals(len(blast_task.get_txt_files()), 0)

            # try 1 txt file
            open(os.path.join(blast_task.get_dir(), '4vwx.txt'), 'a').close()
            self.assertEquals(len(blast_task.get_txt_files()), 1)

            # try 1 summary.txt and 1 txt file
            summary_file = os.path.join(blast_task.get_dir(), 'summary.txt')
            open(summary_file, 'a').close()
            self.assertEquals(len(blast_task.get_txt_files()), 1)

            # try multiple txt files
            open(os.path.join(blast_task.get_dir(), '5vwx.txt'), 'a').close()
            self.assertEquals(len(blast_task.get_txt_files()), 2)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_uploadable_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # test where directory doesn't even exist
            params = D3RParameters()
            task = BlastNFilterTask(temp_dir, params)
            self.assertEqual(task.get_uploadable_files(), [])

            # test on empty dir
            task.create_dir()

            # test with blastn filter log
            logfile = os.path.join(task.get_dir(),
                                   BlastNFilterTask.BLASTNFILTER_LOG)
            open(logfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 1)
            self.assertEqual(flist[0], logfile)

            # test with dockable.xslx
            dockable = os.path.join(task.get_dir(),
                                    BlastNFilterTask.DOCKABLE_XSLX)
            open(dockable, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            flist.index(dockable)

            # test with summary.txt
            summary = os.path.join(task.get_dir(),
                                   BlastNFilterTask.SUMMARY_TXT)
            open(summary, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 3)
            flist.index(summary)

            # test with 1 txt file
            txt_one = os.path.join(task.get_dir(),
                                   '1fcz.txt')
            open(txt_one, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 4)
            flist.index(txt_one)

            # test with 2 txt files
            txt_two = os.path.join(task.get_dir(),
                                   '4asd.txt')
            open(txt_two, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 5)
            flist.index(txt_two)

            # test with additional stderr/stdout files
            errfile = os.path.join(task.get_dir(),
                                   'blastnfilter.py.stderr')
            open(errfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 6)
            flist.index(errfile)

            outfile = os.path.join(task.get_dir(),
                                   'blastnfilter.py.stdout')
            open(outfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 7)
            flist.index(outfile)
            flist.index(errfile)
            flist.index(txt_two)
            flist.index(txt_one)
            flist.index(summary)
            flist.index(dockable)
            flist.index(logfile)

        finally:
            shutil.rmtree(temp_dir)

    def test_can_run(self):
        tempDir = tempfile.mkdtemp()

        try:
            # try where makeblastdb is not complete
            params = D3RParameters()
            blastTask = BlastNFilterTask(tempDir, params)
            self.assertEqual(blastTask.can_run(), False)

            # try where makeblastdb failed
            blastDb = MakeBlastDBTask(tempDir, params)
            blastDb.create_dir()
            errorFile = os.path.join(blastDb.get_path(),
                                     blastDb.get_dir_name(),
                                     D3RTask.ERROR_FILE)
            open(errorFile, 'a').close()
            self.assertEqual(blastTask.can_run(), False)
            self.assertEqual(blastTask.get_error(),
                             'makeblastdb task has error status')

            # try where data import is not complete
            completeFile = os.path.join(blastDb.get_path(),
                                        blastDb.get_dir_name(),
                                        D3RTask.COMPLETE_FILE)
            open(completeFile, 'a').close()
            self.assertEqual(blastTask.can_run(), False)
            self.assertEqual(blastTask.get_error(), 'dataimport task has ' +
                             'notfound status')

            # try where data import failed
            dataImport = DataImportTask(tempDir, params)
            dataImport.create_dir()
            errorFile = os.path.join(dataImport.get_path(),
                                     dataImport.get_dir_name(),
                                     D3RTask.ERROR_FILE)
            open(errorFile, 'a').close()
            self.assertEqual(blastTask.can_run(), False)
            self.assertEqual(blastTask.get_error(),
                             'dataimport task has error status')

            # try where blast can run
            os.remove(errorFile)
            completeFile = os.path.join(dataImport.get_dir(),
                                        D3RTask.COMPLETE_FILE)
            open(completeFile, 'a').close()
            self.assertEqual(blastTask.can_run(), True)
            self.assertEqual(blastTask.get_error(), None)

            # try where blast exists
            blastTask.create_dir()
            self.assertEqual(blastTask.can_run(), False)
            self.assertEqual(blastTask.get_error(),
                             blastTask.get_dir_name() + ' already exists and' +
                             ' status is unknown')

            # try where blast is complete
            completeFile = os.path.join(blastTask.get_path(),
                                        blastTask.get_dir_name(),
                                        D3RTask.COMPLETE_FILE)
            open(completeFile, 'a').close()
            self.assertEqual(blastTask.can_run(), False)
            self.assertEqual(blastTask.get_error(), None)

        finally:
            shutil.rmtree(tempDir)

    def test_run_with_blast_success_usenewsequencetsv(self):
        temp_dir = tempfile.mkdtemp()

        try:
            params = D3RParameters()
            params.blastnfilter = '/bin/echo'
            params.postanalysis = os.path.join(temp_dir, 'foo.py')
            params.pdbdb = '/pdbdb'
            blasttask = BlastNFilterTask(temp_dir, params)
            blasttask._can_run = True

            txt_file = os.path.join(blasttask.get_dir(), 'summary.txt')

            txt_contents = ('INPUT SUMMARY\\n' +
                            '  sequences:  177\\n' +
                            '  complexes:  149\\n')
            # create fake blastnfilter script that makes csv files
            f = open(params.postanalysis, 'w')
            f.write('#! /usr/bin/env python\n\n')
            f.write('f = open(\'' + txt_file + '\', \'w\')\n')
            f.write('f.write(\'' + txt_contents + '\\n\')\n')
            f.write('f.flush()\nf.close()\n')
            f.flush()
            f.close()
            os.chmod(params.postanalysis, stat.S_IRWXU)

            dtask = DataImportTask(temp_dir, params)
            dtask.create_dir()
            open(dtask.get_sequence_tsv(), 'a').close()

            blasttask.run()
            self.assertEqual(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            self.assertEqual(blasttask.get_error(), None)
            complete_file = os.path.join(blasttask.get_dir(),
                                         D3RTask.COMPLETE_FILE)

            self.assertEqual(os.path.isfile(complete_file), True)

            std_err_file = os.path.join(blasttask.get_dir(),
                                        'echo.stderr')

            self.assertEqual(os.path.isfile(std_err_file), True)

            std_out_file = os.path.join(blasttask.get_dir(),
                                        'echo.stdout')

            dataimport = DataImportTask(temp_dir, params)
            makeblast = MakeBlastDBTask(temp_dir, params)

            f = open(std_out_file, 'r')
            echo_out = f.read().replace('\n', '')
            echo_out.index('--nonpolymertsv ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.NONPOLYMER_TSV
                                        ))
            echo_out.index(' --sequencetsv ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.SEQUENCE_TSV))
            echo_out.index(' --pdbblastdb ' +
                           os.path.join(temp_dir, makeblast.get_dir_name()))
            echo_out.index(' --compinchi ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.COMPINCHI_ICH))
            echo_out.index(' --outdir ' +
                           os.path.join(temp_dir, blasttask.get_dir_name()))
            echo_out.index(' --crystalpH ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.CRYSTALPH_TSV))
            echo_out.index(' --pdbdb /pdbdb ')
            f.close()

            self.assertEqual(os.path.isfile(std_out_file), True)
            self.assertEquals(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            self.assertEquals(os.path.exists(os.path.join(blasttask.get_dir(),
                                                          'foo.py.stderr')),
                              True)
            self.assertEquals(os.path.exists(os.path.join(blasttask.get_dir(),
                                                          'foo.py.stdout')),
                              True)
            res = blasttask.get_email_log().rstrip('\n')
            res.index('/bin/echo')
            res.index('# txt files found: 0')
            res.index('Output from summary.txt')
            res.index('  sequences:  177')
            res.index('  complexes:  149')
            self.assertEqual(res.find(dataimport.get_sequence_tsv() +
                                      ' file not found falling back to ' +
                                      dataimport.get_oldsequence_tsv()), -1)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_with_blast_success_useoldseq_and_postanalysis_fail(self):
        temp_dir = tempfile.mkdtemp()

        try:
            params = D3RParameters()
            params.blastnfilter = '/bin/echo'
            params.postanalysis = os.path.join(temp_dir, 'foo.py')
            params.pdbdb = '/pdbdb'
            blasttask = BlastNFilterTask(temp_dir, params)
            blasttask._can_run = True

            txt_file = os.path.join(blasttask.get_dir(), 'summary.txt')

            txt_contents = ('INPUT SUMMARY\\n' +
                            '  sequences:  177\\n' +
                            '  complexes:  149\\n')
            # create fake blastnfilter script that makes csv files
            f = open(params.postanalysis, 'w')
            f.write('#! /usr/bin/env python\n\n')
            f.write('f = open(\'' + txt_file + '\', \'w\')\n')
            f.write('f.write(\'' + txt_contents + '\\n\')\n')
            f.write('f.flush()\nf.close()\n')
            f.flush()
            f.close()
            os.chmod(params.postanalysis, stat.S_IRWXU)

            blasttask.run()
            self.assertEqual(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            self.assertEqual(blasttask.get_error(), None)
            complete_file = os.path.join(blasttask.get_dir(),
                                         D3RTask.COMPLETE_FILE)

            self.assertEqual(os.path.isfile(complete_file), True)

            std_err_file = os.path.join(blasttask.get_dir(),
                                        'echo.stderr')

            self.assertEqual(os.path.isfile(std_err_file), True)

            std_out_file = os.path.join(blasttask.get_dir(),
                                        'echo.stdout')

            dataimport = DataImportTask(temp_dir, params)
            makeblast = MakeBlastDBTask(temp_dir, params)

            f = open(std_out_file, 'r')
            echo_out = f.read().replace('\n', '')
            echo_out.index('--nonpolymertsv ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.NONPOLYMER_TSV
                                        ))
            echo_out.index(' --sequencetsv ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.OLDSEQUENCE_TSV))
            echo_out.index(' --pdbblastdb ' +
                           os.path.join(temp_dir, makeblast.get_dir_name()))
            echo_out.index(' --compinchi ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.COMPINCHI_ICH))
            echo_out.index(' --outdir ' +
                           os.path.join(temp_dir, blasttask.get_dir_name()))
            echo_out.index(' --crystalpH ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.CRYSTALPH_TSV))
            echo_out.index(' --pdbdb /pdbdb ')
            f.close()

            self.assertEqual(os.path.isfile(std_out_file), True)
            self.assertEquals(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            self.assertEquals(os.path.exists(os.path.join(blasttask.get_dir(),
                                                          'foo.py.stderr')),
                              True)
            self.assertEquals(os.path.exists(os.path.join(blasttask.get_dir(),
                                                          'foo.py.stdout')),
                              True)
            res = blasttask.get_email_log().rstrip('\n')
            res.index('/bin/echo')
            res.index('# txt files found: 0')
            res.index('Output from summary.txt')
            res.index('  sequences:  177')
            res.index('  complexes:  149')
            res.index(dataimport.get_sequence_tsv() +
                      ' file not found falling back to ' +
                      dataimport.get_oldsequence_tsv())
        finally:
            shutil.rmtree(temp_dir)

    def test_run_with_blast_success_postanalysis_success_no_summary_file(self):
        temp_dir = tempfile.mkdtemp()

        try:
            params = D3RParameters()
            params.blastnfilter = 'true'
            params.postanalysis = '/bin/echo'
            params.pdbdb = '/pdbdb'
            blasttask = BlastNFilterTask(temp_dir, params)
            blasttask._can_run = True
            blasttask.run()
            self.assertEqual(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            self.assertEqual(blasttask.get_error(), None)
            complete_file = os.path.join(blasttask.get_dir(),
                                         D3RTask.COMPLETE_FILE)

            self.assertEqual(os.path.isfile(complete_file), True)

            std_err_file = os.path.join(blasttask.get_dir(),
                                        'echo.stderr')

            self.assertEqual(os.path.isfile(std_err_file), True)

            std_out_file = os.path.join(blasttask.get_dir(),
                                        'echo.stdout')
            dataimport = DataImportTask(temp_dir, params)
            f = open(std_out_file, 'r')
            echo_out = f.read().replace('\n', '')
            echo_out.index('--compinchi ' +
                           os.path.join(temp_dir, dataimport.get_dir_name(),
                                        DataImportTask.COMPINCHI_ICH))

            echo_out.index(' ' + os.path.join(temp_dir,
                                              blasttask.get_dir_name()))
            f.close()

            self.assertEqual(os.path.isfile(std_out_file), True)
            self.assertEquals(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            self.assertEquals(os.path.exists(os.path.join(blasttask.get_dir(),
                                                          'true.stderr')),
                              True)
            self.assertEquals(os.path.exists(os.path.join(blasttask.get_dir(),
                                                          'true.stdout')),
                              True)
            res = blasttask.get_email_log().rstrip('\n')
            res.index('/bin/echo')
            res.index('# txt files found: 0')
            res.index('Output from summary.txt')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_success_with_txt_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            foo_script = os.path.join(temp_dir, 'foo.py')
            params.blastnfilter = foo_script
            params.postanalysis = '/bin/echo'
            params.pdbdb = '/pdbdb'
            blasttask = BlastNFilterTask(temp_dir, params)
            blasttask._can_run = True

            txt_file = os.path.join(blasttask.get_dir(), '4za4.txt')

            txt_contents = ('query, 4za4\\n' +
                            'ph, 7.4\\n' +
                            'ligand, 4LU\\n\\n' +
                            'inchi, InChI=1S/C22H29N4O9P/c1-10-7-12-16-' +
                            '15(11(10)2)22(3,4)5-6-25(16)17-19(23-21(31)' +
                            '24-20(17)30)26(12)8-13(27)18(29)14(28)9-35-' +
                            '36(32,33)34/h6-7,13-14,18,27-29H,5,8-9H2,1-' +
                            '4H3,(H3-,23,24,30,31,32,33,34)/p+1/t13-,14+' +
                            ',18-/m0/s1\\n'
                            'largest, 4zz3, 4PP\\n' +
                            'smallest, 3ax3, 4LP\\n' +
                            'holo, 2ax1, XDN\\n' +
                            'apo, 2ll3, GSS\\n')

            # create fake blastnfilter script that makes csv files
            f = open(foo_script, 'w')
            f.write('#! /usr/bin/env python\n\n')
            f.write('f = open(\'' + txt_file + '\', \'w\')\n')
            f.write('f.write(\'' + txt_contents + '\\n\')\n')
            f.write('f.flush()\nf.close()\n')
            f.flush()
            f.close()
            os.chmod(foo_script, stat.S_IRWXU)

            blasttask.run()
            self.assertEqual(blasttask.get_error(), None)
            self.assertEqual(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            complete_file = os.path.join(blasttask.get_dir(),
                                         D3RTask.COMPLETE_FILE)

            self.assertEqual(os.path.isfile(complete_file), True)

            std_err_file = os.path.join(blasttask.get_dir(),
                                        'foo.py.stderr')

            self.assertEqual(os.path.isfile(std_err_file), True)

            std_out_file = os.path.join(blasttask.get_dir(),
                                        'foo.py.stdout')

            self.assertEqual(os.path.isfile(std_out_file), True)

            res = blasttask.get_email_log().rstrip('\n')
            res.index('/foo.py')
            res.index('# txt files found: 1')
            res.index('Output from summary.txt')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_success_with_txt_file_and_timeout_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            foo_script = os.path.join(temp_dir, 'foo.py')
            params.blastnfilter = foo_script
            params.postanalysis = '/bin/echo'
            params.pdbdb = '/pdbdb'
            params.blastnfiltertimeout = 50
            blasttask = BlastNFilterTask(temp_dir, params)
            blasttask._can_run = True

            txt_file = os.path.join(blasttask.get_dir(), '4za4.txt')

            txt_contents = ('query, 4za4\\n' +
                            'ph, 7.4\\n' +
                            'ligand, 4LU\\n\\n' +
                            'inchi, InChI=1S/C22H29N4O9P/c1-10-7-12-16-' +
                            '15(11(10)2)22(3,4)5-6-25(16)17-19(23-21(31)' +
                            '24-20(17)30)26(12)8-13(27)18(29)14(28)9-35-' +
                            '36(32,33)34/h6-7,13-14,18,27-29H,5,8-9H2,1-' +
                            '4H3,(H3-,23,24,30,31,32,33,34)/p+1/t13-,14+' +
                            ',18-/m0/s1\\n'
                            'largest, 4zz3, 4PP\\n' +
                            'smallest, 3ax3, 4LP\\n' +
                            'holo, 2ax1, XDN\\n' +
                            'apo, 2ll3, GSS\\n')

            # create fake blastnfilter script that makes csv files
            f = open(foo_script, 'w')
            f.write('#! /usr/bin/env python\n\n')
            f.write('f = open(\'' + txt_file + '\', \'w\')\n')
            f.write('f.write(\'' + txt_contents + '\\n\')\n')
            f.write('f.flush()\nf.close()\n')
            f.flush()
            f.close()
            os.chmod(foo_script, stat.S_IRWXU)

            blasttask.run()
            self.assertEqual(blasttask.get_error(), None)
            self.assertEqual(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            complete_file = os.path.join(blasttask.get_dir(),
                                         D3RTask.COMPLETE_FILE)

            self.assertEqual(os.path.isfile(complete_file), True)

            std_err_file = os.path.join(blasttask.get_dir(),
                                        'foo.py.stderr')

            self.assertEqual(os.path.isfile(std_err_file), True)

            std_out_file = os.path.join(blasttask.get_dir(),
                                        'foo.py.stdout')

            self.assertEqual(os.path.isfile(std_out_file), True)

            res = blasttask.get_email_log().rstrip('\n')
            res.index('/foo.py')
            res.index('# txt files found: 1')
            res.index('Output from summary.txt')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fail_due_to_time_but_with_txt_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            foo_script = os.path.join(temp_dir, 'foo.py')
            params.blastnfilter = foo_script
            params.postanalysis = '/bin/echo'
            params.pdbdb = '/pdbdb'
            params.blastnfiltertimeout = 1
            blasttask = BlastNFilterTask(temp_dir, params)
            blasttask._can_run = True

            txt_file = os.path.join(blasttask.get_dir(), '4za4.txt')

            txt_contents = ('query, 4za4\\n' +
                            'ph, 7.4\\n' +
                            'ligand, 4LU\\n\\n' +
                            'inchi, InChI=1S/C22H29N4O9P/c1-10-7-12-16-' +
                            '15(11(10)2)22(3,4)5-6-25(16)17-19(23-21(31)' +
                            '24-20(17)30)26(12)8-13(27)18(29)14(28)9-35-' +
                            '36(32,33)34/h6-7,13-14,18,27-29H,5,8-9H2,1-' +
                            '4H3,(H3-,23,24,30,31,32,33,34)/p+1/t13-,14+' +
                            ',18-/m0/s1\\n'
                            'largest, 4zz3, 4PP\\n' +
                            'smallest, 3ax3, 4LP\\n' +
                            'holo, 2ax1, XDN\\n' +
                            'apo, 2ll3, GSS\\n')

            # create fake blastnfilter script that makes csv files
            f = open(foo_script, 'w')
            f.write('#! /usr/bin/env python\n\n')
            f.write('import time\n')
            f.write('f = open(\'' + txt_file + '\', \'w\')\n')
            f.write('f.write(\'' + txt_contents + '\\n\')\n')
            f.write('f.flush()\nf.close()\n')
            f.write('time.sleep(360)\n')
            f.flush()
            f.close()
            os.chmod(foo_script, stat.S_IRWXU)

            blasttask.run()
            self.assertEqual(blasttask.get_error(), None)
            self.assertEqual(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            complete_file = os.path.join(blasttask.get_dir(),
                                         D3RTask.COMPLETE_FILE)

            self.assertEqual(os.path.isfile(complete_file), True)

            std_err_file = os.path.join(blasttask.get_dir(),
                                        'foo.py.stderr')

            self.assertEqual(os.path.isfile(std_err_file), True)

            std_out_file = os.path.join(blasttask.get_dir(),
                                        'foo.py.stdout')

            self.assertEqual(os.path.isfile(std_out_file), True)

            res = blasttask.get_email_log().rstrip('\n')
            res.index('/foo.py')
            res.index('# txt files found: 1')
            res.index('Output from summary.txt')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_with_error(self):
        temp_dir = tempfile.mkdtemp()

        try:
            params = D3RParameters()
            params.blastnfilter = 'false'
            params.postanalysis = 'true'
            params.pdbdb = '/pdbdb'
            blasttask = BlastNFilterTask(temp_dir, params)
            blasttask._can_run = True
            blasttask.run()
            self.assertEqual(blasttask.get_status(), D3RTask.COMPLETE_STATUS)
            self.assertEqual(blasttask.get_error(), None)
            complete_file = os.path.join(blasttask.get_dir(),
                                         D3RTask.COMPLETE_FILE)

            self.assertEqual(os.path.isfile(complete_file), True)
            error_file = os.path.join(blasttask.get_dir(),
                                      D3RTask.ERROR_FILE)

            self.assertEqual(os.path.isfile(error_file), False)

            std_err_file = os.path.join(blasttask.get_dir(),
                                        'false.stderr')

            self.assertEqual(os.path.isfile(std_err_file), True)

            std_out_file = os.path.join(blasttask.get_dir(),
                                        'false.stdout')

            self.assertEqual(os.path.isfile(std_out_file), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_with_exception(self):
        temp_dir = tempfile.mkdtemp()

        try:
            params = D3RParameters()
            params.blastnfilter = 'falseasdfasdf'
            params.postanalysis = 'true'
            params.pdbdb = '/pdbdb'
            blasttask = BlastNFilterTask(temp_dir, params)
            blasttask._can_run = True
            blasttask.run()
            self.assertEqual(blasttask.get_status(), D3RTask.ERROR_STATUS)
            self.assertEqual(blasttask.get_error().startswith('Caught'), True)
            self.assertNotEqual(blasttask.get_error(), None)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_with_can_run_already_set_false(self):
        params = D3RParameters()
        params.blastnfilter = 'false'
        params.postanalysis = 'false'
        params.pdbdb = '/pdbdb'
        blasttask = BlastNFilterTask(None, params)
        blasttask._can_run = False
        blasttask.run()

    def test_parse_blastnfilter_output_for_hit_stats(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            blasttask = BlastNFilterTask(temp_dir, params)

            blasttask.create_dir()

            # no summary.txt file
            self.assertEquals(blasttask
                              ._parse_blastnfilter_output_for_hit_stats(),
                              '\n# txt files found: 0\n\nOutput from ' +
                              'summary.txt\n')

            csv_file = os.path.join(blasttask.get_dir(), '4zyc.txt')
            f = open(csv_file, 'w')
            f.write('NEED TO PUT REAL DATA IN HERE\n')
            f.flush()
            f.close()
            self.assertEquals(blasttask
                              ._parse_blastnfilter_output_for_hit_stats(),
                              '\n# txt files found: 1\n\nOutput from ' +
                              'summary.txt\n')

            csv_file = os.path.join(blasttask.get_dir(), '4qqq.txt')
            f = open(csv_file, 'w')
            f.write('NEED TO PUT REAL DATA IN HERE\n')
            f.flush()
            f.close()
            res = blasttask._parse_blastnfilter_output_for_hit_stats()\
                .rstrip('\n')
            res.index('# txt files found: 2')
            res.index('Output from summary.txt')

            csv_file = os.path.join(blasttask.get_dir(), '4abc.txt')
            f = open(csv_file, 'w')
            f.write('NEED TO PUT REAL DATA IN HERE\n')
            f.flush()
            f.close()
            res = blasttask._parse_blastnfilter_output_for_hit_stats()\
                .rstrip('\n')
            res.index('# txt files found: 3')
            res.index('Output from summary.txt')

        finally:
            shutil.rmtree(temp_dir)

    def test_get_blastnfilter_summary(self):
        """Tests BlastNFilterTask.get_blastnfilter_summary call
        """
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            blasttask = BlastNFilterTask(temp_dir, params)
            summary = blasttask.get_blastnfilter_summary()
            self.assertEqual(summary.get_complexes(), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_blastnfilter_summary_none_for_path(self):
        summary = BlastNFilterSummary(None)
        self.assertEqual(summary.get_complexes(), 0)
        self.assertEqual(summary.get_dockable_complexes(), 0)
        self.assertEqual(summary.get_dockable_monomers(), 0)
        self.assertEqual(summary.get_targets_found(), 0)
        self.assertEqual(summary.get_week_number(), '0')
        self.assertEqual(summary.get_year(), '0')
        self.assertEqual(summary.get_csv(), '0,0,0,0,0,0')

    def test_blastnfilter_summary_no_summary_file(self):
        summary = BlastNFilterSummary('/foo')
        self.assertEqual(summary.get_complexes(), 0)
        self.assertEqual(summary.get_dockable_complexes(), 0)
        self.assertEqual(summary.get_dockable_monomers(), 0)
        self.assertEqual(summary.get_targets_found(), 0)
        self.assertEqual(summary.get_week_number(), '0')
        self.assertEqual(summary.get_year(), '0')
        self.assertEqual(summary.get_csv(), '0,0,0,0,0,0')

    def test_blastnfilter_summary_getters_and_setters(self):
        summary = BlastNFilterSummary('/foo')

        summary.set_complexes(5)
        summary.set_dockable_complexes(6)
        summary.set_dockable_monomers(7)
        summary.set_targets_found(8)

        self.assertEqual(summary.get_complexes(), 5)

        self.assertEqual(summary.get_dockable_complexes(), 6)
        self.assertEqual(summary.get_dockable_monomers(), 7)
        self.assertEqual(summary.get_targets_found(), 8)
        self.assertEqual(summary.get_csv(), '0,0,5,6,7,8')

    def test_blastnfilter_summary_week_and_year(self):

        blast = BlastNFilterTask('/foo', D3RParameters())
        summary = BlastNFilterSummary('/foo/2018/dataset.week.4'
                                      '/' + blast.get_dir_name())
        self.assertEqual(summary.get_week_number(), '4')
        self.assertEqual(summary.get_year(), '2018')
        self.assertEqual(summary.get_csv(), '4,2018,0,0,0,0')

    def test_blastnfilter_summary_parse_summary_file_on_emptyfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            open(os.path.join(temp_dir,
                              BlastNFilterTask.SUMMARY_TXT), 'a').close()
            summary = BlastNFilterSummary(temp_dir)
            self.assertEqual(summary.get_csv(), '0,0,0,0,0,0')
        finally:
            shutil.rmtree(temp_dir)

    def test_blastnfilter_summary_parse_summary_file_on_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            os.mkdir(os.path.join(temp_dir,
                                  BlastNFilterTask.SUMMARY_TXT))
            summary = BlastNFilterSummary(temp_dir)
            self.assertEqual(summary.get_csv(), '0,0,0,0,0,0')
        finally:
            shutil.rmtree(temp_dir)

    def test_blastnfilter_summary_parse_summary_file_on_validfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            f = open(os.path.join(temp_dir,
                                  BlastNFilterTask.SUMMARY_TXT), 'w')
            f.write('INPUT SUMMARY\n')
            f.write('  entries:                             221\n')
            f.write('  complexes:                           178\n')
            f.write('  dockable complexes:                   95\n')
            f.write('  monomers:                            145\n')
            f.write('  dockable monomers:                    71\n')
            f.write('  multimers:                            76\n')
            f.write('  dockable multimers:                   24\n\n')

            f.write('FILTERING CRITERIA\n')
            f.write('  No. of query sequences           <=    1\n')
            f.write('  No. of dockable ligands           =    1\n')
            f.write('  Percent identity                 >=    0.95\n')
            f.write('  Percent Coverage                 >=    0.9\n')
            f.write('  No. of hit sequences             <=    4\n')
            f.write('  Structure determination method:        '
                    '  x-ray diffraction\n\n')

            f.write('OUTPUT SUMMARY\n')
            f.write('  Targets found:                        67\n')
            f.write('  Target: 5fz7|Sequences: 1|Hits: 94|Candidates: 17|'
                    'Elected:4|PDBids: 5fz7,5fyz,5a3p,5a1f\n')

            f.flush()
            f.close()

            summary = BlastNFilterSummary(temp_dir)
            self.assertEqual(summary.get_csv(), '0,0,178,95,71,67')
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python

import sys
import os
import argparse
import logging

import d3r
from d3r.celpp import util
from d3r.celpp.task import D3RParameters
from d3r.celpp.blastnfilter import BlastNFilterTask

# create logger
logger = logging.getLogger('d3r.celppreports')
LOG_FORMAT = "%(asctime)-15s %(levelname)s %(name)s %(message)s"

BLASTNFILTER_SUMMARY_CSV = 'blastnfilter.summary.csv'


def _setup_logging(theargs):
    """Sets up the logging for application

    Loggers are setup for:
    d3r.celppreports
    d3r.celpp.blastnfilter
    d3r.celpp.dataimport
    d3r.celpp.glide
    d3r.celpp.makeblastdb
    d3r.celpp.proteinligprep
    d3r.celpp.evaluation
    d3r.celpp.task
    d3r.celpp.util
    d3r.celpp.uploader

    NOTE:  If new modules are added please add their loggers to this
    function

    The loglevel is set by theargs.loglevel and the format is set
    by LOG_FORMAT set at the top of this module.
    :param: theargs should have .loglevel set to one of the
    following strings: DEBUG, INFO, WARNING, ERROR, CRITICAL
    """
    # TODO This method should be moved into util
    theargs.logformat = LOG_FORMAT
    theargs.numericloglevel = logging.NOTSET
    if theargs.loglevel == 'DEBUG':
        theargs.numericloglevel = logging.DEBUG
    if theargs.loglevel == 'INFO':
        theargs.numericloglevel = logging.INFO
    if theargs.loglevel == 'WARNING':
        theargs.numericloglevel = logging.WARNING
    if theargs.loglevel == 'ERROR':
        theargs.numericloglevel = logging.ERROR
    if theargs.loglevel == 'CRITICAL':
        theargs.numericloglevel = logging.CRITICAL

    logger.setLevel(theargs.numericloglevel)
    logging.basicConfig(format=theargs.logformat)

    # There should be a line below for every package aka .py file
    # under celpp module
    logging.getLogger('d3r.celpp.blastnfilter')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.dataimport').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.glide').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.makeblastdb')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.proteinligprep')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.evaluation').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.task').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.util').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.filetransfer')\
        .setLevel(theargs.numericloglevel)


def generate_reports(theargs):
    """Generates reports
    """
    celpp_years = util.get_all_celpp_years(theargs.celppdir)

    if theargs.outdir is None:
        raise Exception('--outdir must be set')

    if not os.path.isdir(theargs.outdir):
        os.makedirs(theargs.outdir)

    f = open(os.path.join(theargs.outdir, BLASTNFILTER_SUMMARY_CSV), 'w')
    f.write('Week #, Year, Complexes, Dockable complexes, Dockable monomers, '
            'Targets Found\n')
    for year in celpp_years:
        logger.info('Examining year ' + year)
        for week in util.get_all_celpp_weeks(os.path.join(theargs.celppdir,
                                                          year)):
            logger.debug('Examining week ' + week)
            the_dir = os.path.join(theargs.celppdir, year,
                                   util.DATA_SET_WEEK_PREFIX + week)
            blast = BlastNFilterTask(the_dir, theargs)
            summary = blast.get_blastnfilter_summary()
            f.write(summary.get_csv() + '\n')
    f.flush()
    f.close()


def _parse_arguments(desc, args):
    """Parses command line arguments using argparse.
    """
    parsed_arguments = D3RParameters()

    help_formatter = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=help_formatter)
    parser.add_argument("--outdir",
                        help='Directory to write output reports to ')
    parser.add_argument("celppdir",
                        help='Directory where celpp yearly runs reside')
    parser.add_argument("--log", dest="loglevel", choices=['DEBUG',
                        'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
                        help="Set the logging level",
                        default='WARNING')
    parser.add_argument('--version', action='version',
                        version=('%(prog)s ' + d3r.__version__))
    return parser.parse_args(args, namespace=parsed_arguments)


def main(args):
    """Main entry into script
    :param args: Should be set to sys.argv which contains list of arguments
                 passed on commandline to script with the script name
                 being the first argument
    """
    desc = """
              Generates reports on CELPP processing pipeline.

              This will examine a directory containing outputs from
              CELPP processing pipeline and generate reports about
              those runs.

              At the moment only one report is generated, which summarizes
              candidates molecules that passed the blastnfilter stage.  The
              data is obtained by parsing the summary.txt file located
              within the stage.2.blastnfilter directory.

              This report is saved under --outdir path and is named
              blastnfilter.summary.csv

              This comma separated variable file contains Complexes,
              Dockable complexes, Dockable monomers, and Targets Found
              for each week found in the CELPP directory passed into this
              program.

              Example content of blastnfilter.summary.csv:

              Week #, Year, Complexes, Dockable complexes, Dockable monomers,
              Targets Found
              4,2016,178,95,71,67
              5,2016,120,80,60,30
              .
              .
              .

              """

    theargs = _parse_arguments(desc, args[1:])
    theargs.program = args[0]
    theargs.version = d3r.__version__

    _setup_logging(theargs)

    try:
        generate_reports(theargs)
        return 0
    except Exception:
        logger.exception("Error caught exception")
        return 2

if __name__ == '__main__':  # pragma: no cover
    sys.exit(main(sys.argv))
